@@ -35,12 +35,22 @@ extern int GUIDialogs;
 extern int SevenBit;
 extern int WeirdScroll;
 extern int LoadDesktopMode;
+extern char HelpCommand[128];
+extern char *ConfigSourcePath;
+extern char ConfigFileName[MAXPATH];
+extern int IgnoreBufferList;
+extern char CvsCommand[256];
+extern char CvsLogMode[32];
+extern int ReassignModelIds;
+extern int RecheckReadOnly;
+extern char XShellCommand[256];
 
+const char *GetGUICharacters(const char *which, const char *defChars);
 int LoadConfig(int argc, char **argv, char *CfgFileName);
-int GetIndentMode(char *Str);
-int GetHilitMode(char *Str);
+int GetIndentMode(const char *Str);
+int GetHilitMode(const char *Str);
 int UseDefaultConfig();
-int AddCRegexp(int file, int line, int msg, char *regexp);
+int AddCRegexp(int file, int line, int msg, const char *regexp);
 int LoadFile(char *WhereName, char *CfgName);
 
 #endif
