@@ -38,6 +38,9 @@
 #define CF_HWORDS   121
 #define CF_SUBMENUCOND 122
 #define CF_HWTYPE   123
+#define CF_VARIABLE 124
+#define CF_CONCAT   125
+#define CF_CVSIGNRX 126
 
 #define CF_EOF      254
 
