@@ -17,12 +17,14 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 
+#include "feature.h"
 #include "ftever.h"
 #include "sysdep.h"
-#include "feature.h"
 #include "c_fconfig.h"
 #include "s_files.h"
 #include "c_mode.h"
+#include "console.h"
+#include "c_hilit.h"
 
 #define slen(s) ((s) ? (strlen(s) + 1) : 0)
 
@@ -30,15 +32,16 @@ typedef struct {
     char *Name;
 } ExMacro;
 
-unsigned int CMacros = 0;
-ExMacro *Macros = 0;
-
-FILE *output = 0;
-int lntotal = 0;
-long offset = -1;
-long pos = 0;
-char XTarget[256] = "";
+static unsigned int CMacros = 0;
+static ExMacro *Macros = 0;
 
+static FILE *output = 0;
+static int lntotal = 0;
+static long offset = -1;
+static long pos = 0;
+static char XTarget[MAXPATH] = "";
+static char StartDir[MAXPATH] = "";
+static bool preprocess_only = false;
 
 #include "c_commands.h"
 #include "c_cmdtab.h"
@@ -52,7 +55,7 @@ typedef struct _CurPos {
     const char *name; // filename
 } CurPos;
 
-void cleanup(int xerrno) {
+static void cleanup(int xerrno) {
     if (output)
         fclose(output);
     if (XTarget[0] != 0)
@@ -60,7 +63,7 @@ void cleanup(int xerrno) {
     exit (xerrno);
 }
 
-void Fail(CurPos &cp, const char *s, ...) {
+static void Fail(CurPos &cp, const char *s, ...) {
     va_list ap;
     char msgbuf[1024];
 
@@ -72,21 +75,26 @@ void Fail(CurPos &cp, const char *s, ...) {
     cleanup(1);
 }
 
-int LoadFile(const char *WhereName, const char *CfgName, int Level = 1);
-void DefineWord(const char *w);
-void PutObject(CurPos &cp, int xtag, int xlen, void *obj) {
+static int LoadFile(const char *WhereName, const char *CfgName, int Level = 1);
+static void DefineWord(const char *w);
+
+static void PutObject(CurPos &cp, int xtag, int xlen, void *obj) {
     unsigned char tag = (unsigned char)xtag;
     unsigned short len = (unsigned short)xlen;
     unsigned char l[2];
 
-    l[0] = len & 0xFF;
-    l[1] = (len >> 8) & 0xFF;
-
-    if (fwrite(&tag, 1, 1, output) != 1 ||
-        fwrite(l, 2, 1, output) != 1 ||
-        fwrite(obj, 1, len, output) != len)
+    if (preprocess_only == false)
     {
-        Fail(cp, "Disk full!");
+
+	l[0] = len & 0xFF;
+	l[1] = (len >> 8) & 0xFF;
+
+	if (fwrite(&tag, 1, 1, output) != 1 ||
+	    fwrite(l, 2, 1, output) != 1 ||
+	    fwrite(obj, 1, len, output) != len)
+	{
+	    Fail(cp, "Disk full!");
+	}
     }
     pos += 1 + 2 + len;
     if (offset != -1 && pos >= offset) {
@@ -94,15 +102,15 @@ void PutObject(CurPos &cp, int xtag, int xlen, void *obj) {
     }
 }
 
-void PutNull(CurPos &cp, int xtag) {
+static void PutNull(CurPos &cp, int xtag) {
     PutObject(cp, xtag, 0, 0);
 }
 
-void PutString(CurPos &cp, int xtag, char *str) {
+static void PutString(CurPos &cp, int xtag, char *str) {
     PutObject(cp, xtag, slen(str), str);
 }
 
-void PutNumber(CurPos &cp, int xtag, long num) {
+static void PutNumber(CurPos &cp, int xtag, long num) {
     unsigned long l = num;
     unsigned char b[4];
 
@@ -110,19 +118,23 @@ void PutNumber(CurPos &cp, int xtag, long num) {
     b[1] = (unsigned char)((l >> 8) & 0xFF);
     b[2] = (unsigned char)((l >> 16) & 0xFF);
     b[3] = (unsigned char)((l >> 24) & 0xFF);
-    PutObject(cp, xtag, 4, &b);
+    PutObject(cp, xtag, 4, b);
 }
 
 int main(int argc, char **argv) {
-    char Cur[MAXPATH];
     char Source[MAXPATH];
     char Target[MAXPATH];
-    char *p = argv[1];
-    int n = 1;
+    unsigned char b[4];
+    unsigned long l;
+    int n = 0;
 
-    fprintf(stderr, PROG_CFTE " " VERSION " " COPYRIGHT "\n");
-    if (argc < 2 || argc > 4) {
-        fprintf(stderr, "Usage: " PROG_CFTE " [-o<offset>] config/main.fte [fte-new.cnf]\n");
+    fprintf(stderr, PROG_CFTE " " VERSION "\n" COPYRIGHT "\n");
+    if (argc < 2 || argc > 5) {
+        fprintf(stderr, "Usage: " PROG_CFTE " [-o<offset>] [-p[reprocess]] "
+#ifndef UNIX
+                "config/"
+#endif
+                "main.fte [fte-new.cnf]\n");
         exit(1);
     }
 
@@ -138,7 +150,59 @@ int main(int argc, char **argv) {
 #endif
               );
 
-    if (strncmp(p, "-o", 2) == 0) {
+    // setup defaults
+    strcpy(Source, "");
+    strcpy(Target, "fte-new.cnf");
+    preprocess_only = false;
+    offset = -1;
+
+    // parse arguments
+    for (int i = 1; i < argc; i++)
+    {
+	if (argv[i][0] == '-')
+	{
+	    if ((strcmp(argv[i], "-p") == 0) || (strcmp(argv[i], "-preprocess") == 0))
+	    {
+		preprocess_only = true;
+	    } else
+	    if (strncmp(argv[i], "-o", 2) == 0)
+	    {
+		char *p;
+
+		p = argv[i];
+		p += 2;
+		offset = atol(p);
+	    } else
+	    {
+		fprintf(stderr, "Invalid option '%s'\n", argv[i]);
+		exit(1);
+	    }
+	} else
+	{
+	    switch(n)
+	    {
+	    case 0:
+		strcpy(Source, argv[i]);
+		break;
+
+	    case 1:
+                strcpy(Target, argv[i]);
+		break;
+
+	    default:
+		fprintf(stderr, "Invalid option '%s'\n", argv[i]);
+		exit(1);
+	    }
+            n++;
+	}
+    }
+
+    if (n == 0)
+    {
+	fprintf(stderr, "No configuration file specified\n");
+	exit(1);
+    }
+/*    if (strncmp(p, "-o", 2) == 0) {
         p += 2;
         offset = atol(p);
         n++;
@@ -151,42 +215,60 @@ int main(int argc, char **argv) {
     strcpy(Target, "fte-new.cnf");
     if (n < argc)
         strcpy(Target, argv[n++]);
+	*/
 
     JustDirectory(Target, XTarget);
     Slash(XTarget, 1);
-    sprintf(XTarget + strlen(XTarget), "cfte%ld.tmp", (long)getpid());
-    output = fopen(XTarget, "wb");
-    if (output == 0) {
-        fprintf(stderr, "Cannot create '%s', errno=%d!\n", XTarget, errno);
-        cleanup(1);
+
+    if (preprocess_only == false)
+    {
+	sprintf(XTarget + strlen(XTarget), "cfte%ld.tmp", (long)getpid());
+	output = fopen(XTarget, "wb");
+	if (output == 0) {
+	    fprintf(stderr, "Cannot create '%s', errno=%d!\n", XTarget, errno);
+	    cleanup(1);
+	}
+
+	b[0] = b[1] = b[2] = b[3] = 0;
+
+	if (fwrite(b, sizeof(b), 1, output) != 1) {
+	    fprintf(stderr, "Disk full!");
+	    cleanup(1);
+	}
+
+	l = VERNUM;
+
+	b[0] = (unsigned char)(l & 0xFF);
+	b[1] = (unsigned char)((l >> 8) & 0xFF);
+	b[2] = (unsigned char)((l >> 16) & 0xFF);
+	b[3] = (unsigned char)((l >> 24) & 0xFF);
+
+	if (fwrite(b, 4, 1, output) != 1) {
+	    fprintf(stderr, "Disk full!");
+	    cleanup(1);
+	}
+	pos = 2 * 4;
+
+	fprintf(stderr, "Compiling to '%s'\n", Target);
+    } else
+    {
+        pos = 2 * 4;
     }
+    /*{
+        char PrevDir[MAXPATH];
+        sprintf(PrevDir, "%s/..", Target);
+        ExpandPath(PrevDir, StartDir);
+        Slash(StartDir, 1);
+    }*/
 
-    unsigned char b[4];
-
-    b[0] = b[1] = b[2] = b[3] = 0;
-
-    if (fwrite(&b, 4, 1, output) != 1) {
-        fprintf(stderr, "Disk full!");
-        cleanup(1);
-    }
-
-    unsigned long l = VERNUM;
-
-    b[0] = (unsigned char)(l & 0xFF);
-    b[1] = (unsigned char)((l >> 8) & 0xFF);
-    b[2] = (unsigned char)((l >> 16) & 0xFF);
-    b[3] = (unsigned char)((l >> 24) & 0xFF);
-
-    if (fwrite(b, 4, 1, output) != 1) {
-        fprintf(stderr, "Disk full!");
-        cleanup(1);
-    }
-    pos = 2 * 4;
-
-    fprintf(stderr, "Compiling to '%s'\n", Target);
-    ExpandPath(".", Cur);
-    Slash(Cur, 1);
+    ExpandPath("."
+#ifdef UNIX
+               "."
+#endif
+               , StartDir);
+    Slash(StartDir, 1);
 
+    if (preprocess_only == false)
     {
         CurPos cp;
         char FSource[MAXPATH];
@@ -206,18 +288,23 @@ int main(int argc, char **argv) {
         PutString(cp, CF_STRING, FSource);
     }
 
-    if (LoadFile(Cur, Source, 0) != 0) {
+    if (LoadFile(StartDir, Source, 0) != 0) {
         fprintf(stderr, "\nCompile failed\n");
         cleanup(1);
     }
 
+    if (preprocess_only == true)
+    {
+        return 0;
+    }
+
     l = CONFIG_ID;
     b[0] = (unsigned char)(l & 0xFF);
     b[1] = (unsigned char)((l >> 8) & 0xFF);
     b[2] = (unsigned char)((l >> 16) & 0xFF);
     b[3] = (unsigned char)((l >> 24) & 0xFF);
     fseek(output, 0, SEEK_SET);
-    fwrite(&b, 4, 1, output);
+    fwrite(b, 4, 1, output);
     fclose(output);
 
     if (unlink(Target) != 0 && errno != ENOENT) {
@@ -248,7 +335,7 @@ typedef struct _OrdLookup {
     int num;
 } OrdLookup;
 
-OrdLookup mode_num[] = {
+static const OrdLookup mode_num[] = {
 MODE_BFI(AutoIndent),
 MODE_BFI(Insert),
 MODE_BFI(DrawOn),
@@ -292,10 +379,13 @@ MODE_BFI(InsertKillBlock),
 MODE_BFI(UndoMoves),
 MODE_BFI(DetectLineSep),
 MODE_BFI(TrimOnSave),
+MODE_BFI(SaveBookmarks),
+MODE_BFI(HilitTags),
+MODE_BFI(ShowBookmarks),
 { 0, 0 },
 };
 
-OrdLookup mode_string[] = {
+static const OrdLookup mode_string[] = {
 MODE_BFI(Colorizer),
 MODE_BFI(IndentMode),
 MODE_BFS(RoutineRegexp),
@@ -312,7 +402,7 @@ MODE_BFI(EventMap),
 { 0, 0 },
 };
 
-OrdLookup global_num[] = {
+static const OrdLookup global_num[] = {
 #ifdef CONFIG_INDENT_C
 MODE_FLG(C_Indent),
 MODE_FLG(C_BraceOfs),
@@ -327,6 +417,7 @@ MODE_FLG(C_FirstLevelWidth),
 MODE_FLG(C_FirstLevelIndent),
 MODE_FLG(C_Continuation),
 MODE_FLG(C_ParenDelta),
+MODE_FLG(FunctionUsesContinuation),
 #endif
 #ifdef CONFIG_INDENT_REXX
 MODE_FLG(REXX_Indent),
@@ -359,31 +450,38 @@ MODE_FLG(PMDisableAccel),
 MODE_FLG(SevenBit),
 MODE_FLG(WeirdScroll),
 MODE_FLG(LoadDesktopMode),
+MODE_FLG(IgnoreBufferList),
+MODE_FLG(ReassignModelIds),
+MODE_FLG(RecheckReadOnly),
 { 0, 0 },
 };
 
-OrdLookup global_string[] = {
+static const OrdLookup global_string[] = {
 MODE_FLG(DefaultModeName),
 MODE_FLG(CompletionFilter),
 MODE_FLG(PrintDevice),
 MODE_FLG(CompileCommand),
 MODE_FLG(WindowFont),
 MODE_FLG(HelpCommand),
+MODE_FLG(GUICharacters),
+MODE_FLG(CvsCommand),
+MODE_FLG(CvsLogMode),
+MODE_FLG(XShellCommand),
 { 0, 0 },
 };
 
-OrdLookup event_string[] = {
+static const OrdLookup event_string[] = {
 EVENT_FLG(MainMenu),
 EVENT_FLG(LocalMenu),
 { 0, 0 },
 };
 
-OrdLookup colorize_string[] = {
+static const OrdLookup colorize_string[] = {
 COLORIZE_FLG(SyntaxParser),
 { 0, 0 },
 };
 
-OrdLookup hilit_colors[] = {
+static const OrdLookup hilit_colors[] = {
 HILIT_CLR(Normal),
 HILIT_CLR(Keyword),
 HILIT_CLR(String),
@@ -415,7 +513,7 @@ HILIT_CLR(RegexpDelim),
 { 0, 0 },
 };
 
-int Lookup(OrdLookup *where, char *what) {
+static int Lookup(const OrdLookup *where, char *what) {
     int i;
 
     for (i = 0; where[i].Name != 0; i++) {
@@ -464,10 +562,11 @@ int Lookup(OrdLookup *where, char *what) {
 #define K_SUBMENUCOND 19
 #define K_HWTYPE     20
 #define K_COLPALETTE 21
+#define K_CVSIGNRX   22
 
 typedef char Word[64];
 
-OrdLookup CfgKW[] = {
+static const OrdLookup CfgKW[] = {
 { "mode", K_MODE },
 { "eventmap", K_EVENTMAP },
 { "key", K_KEY },
@@ -489,10 +588,11 @@ OrdLookup CfgKW[] = {
 { "h_words", K_HWORDS },
 { "h_wtype", K_HWTYPE },
 { "submenucond", K_SUBMENUCOND },
+{ "CvsIgnoreRx", K_CVSIGNRX },
 { 0, 0 },
 };
 
-OrdLookup CfgVar[] = {
+static const OrdLookup CfgVar[] = {
     { "FilePath", mvFilePath },
     { "FileName", mvFileName },
     { "FileDirectory", mvFileDirectory },
@@ -504,13 +604,14 @@ OrdLookup CfgVar[] = {
     { "Char", mvChar },
     { "Word", mvWord },
     { "Line", mvLine },
+    { "FTEVer", mvFTEVer },
     { 0, 0 },
 };
 
-char **words = 0;
-unsigned int wordCount = 0;
+static char **words = 0;
+static unsigned int wordCount = 0;
 
-int DefinedWord(const char *w) {
+static int DefinedWord(const char *w) {
     if (words == 0 || wordCount == 0)
         return 0;
     for (unsigned int i = 0; i < wordCount; i++)
@@ -519,7 +620,7 @@ int DefinedWord(const char *w) {
     return 0;
 }
 
-void DefineWord(const char *w) {
+static void DefineWord(const char *w) {
     if (!w || !w[0])
         return ;
     if (!DefinedWord(w)) {
@@ -531,13 +632,13 @@ void DefineWord(const char *w) {
     }
 }
 
-int colorCount;
-struct _color {
+static int colorCount;
+static struct _color {
     char *colorName;
     char *colorValue;
 } *colors;
 
-int DefineColor(char *name, char *value) {
+static int DefineColor(char *name, char *value) {
     if (!name || !value)
         return 0;
     colors = (struct _color *)realloc(colors, sizeof (struct _color) * (colorCount + 1));
@@ -551,7 +652,7 @@ int DefineColor(char *name, char *value) {
     return 1;
 }
 
-char *DefinedColor(char *name) {
+static char *DefinedColor(char *name) {
     if (colors == 0 || colorCount == 0)
         return 0;
     for (int i = 0; i < colorCount; i++)
@@ -560,7 +661,7 @@ char *DefinedColor(char *name) {
     return 0;
 }
 
-char *GetColor(CurPos &cp, char *name) {
+static char *GetColor(CurPos &cp, char *name) {
     char *c;
     static char color[4];
 
@@ -592,7 +693,7 @@ char *GetColor(CurPos &cp, char *name) {
     return name;
 }
 
-int GetWord(CurPos &cp, char *w) {
+static int GetWord(CurPos &cp, char *w) {
     char *p = w;
     int len = 0;
 
@@ -611,7 +712,7 @@ int GetWord(CurPos &cp, char *w) {
 }
 
 
-int Parse(CurPos &cp) {
+static int Parse(CurPos &cp) {
     while (cp.c < cp.z) {
         switch (*cp.c) {
 #ifndef UNIX
@@ -620,90 +721,9 @@ int Parse(CurPos &cp) {
         case '#':
             while (cp.c < cp.z && *cp.c != '\n') cp.c++;
             break;
-        case '%':
-            cp.c++;
-            if (cp.c + 7 < cp.z && strncmp(cp.c, "define(", 7) == 0) {
-                Word w;
-                cp.c += 7;
-
-                while (cp.c < cp.z && *cp.c != ')') {
-                    GetWord(cp, w);
-                    //printf("define '%s'\n", w);
-                    DefineWord(w);
-                    if (cp.c < cp.z && *cp.c != ',' && *cp.c != ')' )
-                        Fail(cp, "unexpected: %c", cp.c[0]);
-                    if (cp.c < cp.z && *cp.c == ',')
-                        cp.c++;
-                }
-                cp.c++;
-/*            } else if (cp.c + 6 && strcmp(cp.c, "undef(", 6) == 0) {
-                Word w;
-                cp.c += 6;
-
-                while (cp.c < cp.z && *cp.c != ')') {
-                    GetWord(cp, w);
-                    UndefWord(w);
-                }*/
-            } else if (cp.c + 3 < cp.z && strncmp(cp.c, "if(", 3) == 0) {
-                Word w;
-                int wasWord = 0;
-                cp.c += 3;
-
-                while (cp.c < cp.z && *cp.c != ')') {
-                    int neg = 0;
-                    if (*cp.c == '!') {
-                        cp.c++;
-                        neg = 1;
-                    }
-                    GetWord(cp, w);
-                    if (DefinedWord(w))
-                        wasWord = 1;
-                    if (neg)
-                        wasWord = wasWord ? 0 : 1;
-                    /*if (wasWord)
-                        printf("yes '%s'\n", w);
-                    else
-                        printf("not '%s'\n", w);*/
-
-                    if (cp.c < cp.z && *cp.c != ',' && *cp.c != ')' )
-                        Fail(cp, "unexpected: %c", cp.c[0]);
-                    if (cp.c < cp.z && *cp.c == ',')
-                        cp.c++;
-                }
-                cp.c++;
-                if (!wasWord) {
-                    int nest = 1;
-                    while (cp.c < cp.z) {
-                        if (*cp.c == '\n') {
-                            cp.line++;
-                            lntotal++;
-                        } else if (*cp.c == '%') {
-                            if (cp.c + 6 < cp.z &&
-                                strncmp(cp.c, "%endif", 6) == 0)
-                            {
-                                cp.c += 6;
-                                if (--nest == 0)
-                                    break;
-                            }
-                            if (cp.c + 4 < cp.z &&
-                                strncmp(cp.c, "%if", 4) == 0)
-                            {
-                                cp.c += 4;
-                                ++nest;
-                            }
-                        }
-                        cp.c++;
-                    }
-                }
-            } else if (cp.c + 5 < cp.z && strncmp(cp.c, "endif", 5) == 0) {
-                cp.c += 5;
-            }
-            if (cp.c < cp.z && *cp.c != '\n' && *cp.c != '\r')
-                Fail(cp, "syntax error %30.30s", cp.c);
-            break;
         case '\n':
             cp.line++;
-            lntotal++;
+	    lntotal++;
         case ' ':
         case '\t':
         case '\r':
@@ -738,7 +758,7 @@ int Parse(CurPos &cp) {
     return P_EOF;
 }
 
-void GetOp(CurPos &cp, int what) {
+static void GetOp(CurPos &cp, int what) {
     switch (what) {
     case P_COMMA:
     case P_OPENBRACE:
@@ -754,10 +774,11 @@ void GetOp(CurPos &cp, int what) {
     }
 }
 
-char *GetString(CurPos &cp) {
+static char *GetString(CurPos &cp) {
     char c = *cp.c;
     char *p = cp.c;
     char *d = cp.c;
+    int n;
 
     if (c == '[') c = ']';
 
@@ -777,15 +798,31 @@ char *GetString(CurPos &cp) {
                 case 'b': *cp.c = '\b'; break;
                 case 'v': *cp.c = '\v'; break;
                 case 'a': *cp.c = '\a'; break;
+                case 'x':
+                    cp.c++;
+                    if (cp.c == cp.z) return 0;
+                    if (*cp.c >= '0' && *cp.c <= '9') n = *cp.c - '0';
+                    else if (*cp.c >='a' && *cp.c <= 'f') n = *cp.c - 'a' + 10;
+                    else if (*cp.c >='A' && *cp.c <= 'F') n = *cp.c - 'A' + 10;
+                    else return 0;
+                    cp.c++;
+                    if (cp.c == cp.z) cp.c--;
+                    else if (*cp.c >= '0' && *cp.c <= '9') n = n * 16 + *cp.c - '0';
+                    else if (*cp.c >= 'a' && *cp.c <= 'f') n = n * 16 + *cp.c - 'a' + 10;
+                    else if (*cp.c >= 'A' && *cp.c <= 'F') n = n * 16 + *cp.c - 'A' + 10;
+                    else cp.c--;
+                    *cp.c = n;
+                    break;
                 }
             }
         } else if (*cp.c == c) {
             cp.c++;
             *p = 0;
             return d;
-        } else if (*cp.c == '\n') return 0;
-        if (*cp.c == '\n') cp.line++;
-        if (*cp.c == '\r') {
+        } else if (*cp.c == '\n') {
+            cp.line++;
+            return 0;
+        } else if (*cp.c == '\r') {
             cp.c++;
             if (cp.c == cp.z) return 0;
         }
@@ -794,7 +831,7 @@ char *GetString(CurPos &cp) {
     return 0;
 }
 
-int GetNumber(CurPos &cp) {
+static int GetNumber(CurPos &cp) {
     int value = 0;
     int neg = 0;
 
@@ -809,7 +846,7 @@ int GetNumber(CurPos &cp) {
     return neg ? -value : value;
 }
 
-int CmdNum(const char *Cmd) {
+static int CmdNum(const char *Cmd) {
     unsigned int i;
 
     for (i = 0;
@@ -832,7 +869,7 @@ int NewCommand(const char *Name) {
     return CMacros - 1;
 }
 
-int ParseCommands(CurPos &cp, char *Name) {
+static int ParseCommands(CurPos &cp, char *Name) {
     //if (!Name)
     //    return 0;
     Word cmd;
@@ -897,7 +934,7 @@ int ParseCommands(CurPos &cp, char *Name) {
     return 0;
 }
 
-int ParseConfigFile(CurPos &cp) {
+static int ParseConfigFile(CurPos &cp) {
     Word w = "";
     char *s = 0;
     int p = 0;
@@ -1193,6 +1230,9 @@ int ParseConfigFile(CurPos &cp) {
                                     if (Parse(cp) == P_CLOSEBRACE) break;
                                     if (Parse(cp) != P_STRING) Fail(cp, "String expected");
                                     kname = GetString(cp);
+
+                                    if (strlen(kname) >= CK_MAXLEN) Fail(cp, "Keyword name is too long");
+
                                     PutString(cp, CF_STRING, kname);
 
                                     if (Parse(cp) != P_COMMA)
@@ -1269,6 +1309,7 @@ int ParseConfigFile(CurPos &cp) {
                                 //if (strchr(opts, '!')) match_opts |= MATCH_NEGATE;
                                 if (strchr(opts, 'q')) match_opts |= MATCH_QUOTECH;
                                 if (strchr(opts, 'Q')) match_opts |= MATCH_QUOTEEOL;
+                                if (strchr(opts, 'x')) match_opts |= MATCH_REGEXP;
 
                                 if (Parse(cp) != P_COMMA) Fail(cp, "',' expected");
                                 GetOp(cp, P_COMMA);
@@ -1279,7 +1320,7 @@ int ParseConfigFile(CurPos &cp) {
 
                                 PutNumber(cp, CF_INT, match_opts);
                                 PutNumber(cp, CF_INT, cidx);
-                                PutString(cp, CF_STRING, match);
+                                PutString(cp, match_opts & MATCH_REGEXP ? CF_REGEXP : CF_STRING, match);
 
                                 if (Parse(cp) != P_CLOSEBRACE) Fail(cp, "'}' expected");
                                 GetOp(cp, P_CLOSEBRACE);
@@ -1566,6 +1607,20 @@ int ParseConfigFile(CurPos &cp) {
                                 GetOp(cp, P_EOS);
                             }
                             break;
+                        case K_CVSIGNRX:
+                            {
+                                char *regexp;
+
+                                if (Parse(cp) != P_ASSIGN) Fail(cp, "'=' expected");
+                                GetOp(cp, P_ASSIGN);
+                                if (Parse(cp) != P_STRING) Fail(cp, "String expected");
+                                regexp = GetString(cp);
+                                PutNull(cp, CF_CVSIGNRX);
+                                PutString(cp, CF_REGEXP, regexp);
+                                if (Parse(cp) != P_EOS) Fail(cp, "';' expected");
+                                GetOp(cp, P_EOS);
+                            }
+                            break;
                         default:  // mode::
                             if (Parse(cp) != P_ASSIGN) Fail(cp, "'=' expected");
                             GetOp(cp, P_ASSIGN);
@@ -1627,7 +1682,8 @@ int ParseConfigFile(CurPos &cp) {
                         if (Parse(cp) != P_STRING) Fail(cp, "String expected");
                         svalue = GetString(cp);
                         svalue = GetColor(cp, svalue);
-                        assert(DefineColor(sname, svalue) == 1);
+                        if (DefineColor(sname, svalue) != 1)
+                            Fail(cp, "DefineColor failed\n");
                         if (Parse(cp) != P_CLOSEBRACE) Fail(cp, "'}' expected");
                         GetOp(cp, P_CLOSEBRACE);
                         if (Parse(cp) != P_COMMA)
@@ -1661,13 +1717,158 @@ int ParseConfigFile(CurPos &cp) {
     }
 }
 
-int LoadFile(const char *WhereName, const char *CfgName, int Level) {
+static int PreprocessConfigFile(CurPos &cp) {
+    char *wipe = NULL;
+    char *wipe_end = NULL;
+    bool rem_active = false;
+    bool string_open = false;
+
+    while (cp.c < cp.z) {
+	switch(*cp.c)
+	{
+	case '#':
+	    if (string_open == true) break;
+
+	    rem_active = true;
+	    break;
+
+	case '\\':
+            cp.c++;
+            break;
+
+	case '"':
+	case '\'':
+	    if (rem_active == true) break;
+
+	    string_open = !string_open;
+	    break;
+
+	case '%':
+	    if (string_open == true) break;
+	    if (rem_active == true) break;
+
+	    wipe = cp.c;
+            wipe_end = NULL;
+
+            if (cp.c + 8 < cp.z && strncmp(cp.c, "%define(", 8) == 0) {
+                Word w;
+                cp.c += 8;
+
+                while (cp.c < cp.z && *cp.c != ')') {
+                    GetWord(cp, w);
+                    //printf("define '%s'\n", w);
+                    DefineWord(w);
+                    if (cp.c < cp.z && *cp.c != ',' && *cp.c != ')' )
+                        Fail(cp, "unexpected: %c", cp.c[0]);
+                    if (cp.c < cp.z && *cp.c == ',')
+                        cp.c++;
+                }
+		cp.c++;
+/*            } else if (cp.c + 6 && strcmp(cp.c, "undef(", 6) == 0) {
+                Word w;
+                cp.c += 6;
+
+                while (cp.c < cp.z && *cp.c != ')') {
+                    GetWord(cp, w);
+                    UndefWord(w);
+                }*/
+            } else if (cp.c + 4 < cp.z && strncmp(cp.c, "%if(", 4) == 0) {
+                Word w;
+                int wasWord = 0;
+                cp.c += 4;
+
+                while (cp.c < cp.z && *cp.c != ')') {
+                    int neg = 0;
+                    if (*cp.c == '!') {
+                        cp.c++;
+                        neg = 1;
+                    }
+                    GetWord(cp, w);
+                    if (DefinedWord(w))
+                        wasWord = 1;
+                    if (neg)
+                        wasWord = wasWord ? 0 : 1;
+                    /*if (wasWord)
+                        printf("yes '%s'\n", w);
+                    else
+                        printf("not '%s'\n", w);*/
+
+                    if (cp.c < cp.z && *cp.c != ',' && *cp.c != ')' )
+                        Fail(cp, "unexpected: %c", cp.c[0]);
+                    if (cp.c < cp.z && *cp.c == ',')
+                        cp.c++;
+                }
+                cp.c++;
+                if (!wasWord) {
+                    int nest = 1;
+                    while (cp.c < cp.z) {
+                        if (*cp.c == '\n') {
+                            cp.line++;
+                            lntotal++;
+                        } else if (*cp.c == '%') {
+                            if (cp.c + 6 < cp.z &&
+                                strncmp(cp.c, "%endif", 6) == 0)
+                            {
+                                cp.c += 6;
+                                if (--nest == 0)
+                                    break;
+                            }
+                            if (cp.c + 3 < cp.z &&
+                                strncmp(cp.c, "%if", 3) == 0)
+                            {
+                                cp.c += 3;
+                                ++nest;
+                            }
+                        } else if (*cp.c == '#') {
+                            // we really shouldn't process hashed % directives
+                            while( cp.c < cp.z && *cp.c != '\n' ) cp.c++;
+
+                            // workaround to make line numbering correct
+                            cp.line++;
+                            lntotal++;
+                        }
+                        cp.c++;
+                    }
+                }
+            } else if (cp.c + 6 < cp.z && strncmp(cp.c, "%endif", 6) == 0) {
+                cp.c += 6;
+            }
+            if (cp.c < cp.z && *cp.c != '\n' && *cp.c != '\r')
+                Fail(cp, "syntax error %30.30s", cp.c);
+
+	    wipe_end = cp.c;
+
+            // wipe preprocessor macros with space
+	    while (wipe < wipe_end)
+	    {
+                *wipe++ = ' ';
+	    }
+
+	    break;
+
+        case '\n':
+	    cp.line++;
+	    rem_active = false;
+            string_open = false;
+	    break;
+
+	default:
+            break;
+	}
+
+        cp.c++;
+    }
+
+    return 0;
+}
+
+static int LoadFile(const char *WhereName, const char *CfgName, int Level) {
     int fd, rc;
     char *buffer = 0;
     struct stat statbuf;
     CurPos cp;
-    char last[256];
-    char Cfg[256];
+    char last[MAXPATH];
+    char Cfg[MAXPATH];
 
     //fprintf(stderr, "Loading file %s %s\n", WhereName, CfgName);
 
@@ -1676,9 +1877,54 @@ int LoadFile(const char *WhereName, const char *CfgName, int Level) {
     if (IsFullPath(CfgName)) {
         strcpy(Cfg, CfgName);
     } else {
+        // here we will try relative to a number of places.
+        // 1. User's .fte directory.
+        // 2. System's "local config" directory.
+        // 3. Initial file's directory.
+        // 4. Current directory.
+        // This means that a user's directory will always win out,
+        // allowing a given user to always be able to override everything,
+        // followed by a system standard to override anything.
+
+        // #'s 1 and 2 are unix-only.
+#ifdef UNIX
+        // 1. User's .fte directory.
+        char tmp[MAXPATH];
+        sprintf(tmp, "~/.fte/%s", CfgName);
+        ExpandPath(tmp, Cfg);
+        //fprintf(stderr, "Looking for %s\n", Cfg);
+        if (!FileExists(Cfg))
+        {
+            // Okay, now try "local config".
+            sprintf(tmp, "%slocalconfig/%s", StartDir, CfgName);
+            ExpandPath(tmp, Cfg);
+            //fprintf(stderr, "Looking for %s\n", Cfg);
+            if (!FileExists(Cfg))
+            {
+                sprintf(tmp, "%sconfig/%s", StartDir, CfgName);
+                ExpandPath(tmp, Cfg);
+                //fprintf(stderr, "Looking for %s\n", Cfg);
+                if (!FileExists(Cfg))
+                {
+                    sprintf(tmp, "./%s", CfgName);
+                    ExpandPath(tmp, Cfg);
+                    //fprintf(stderr, "Looking for %s\n", Cfg);
+                    if (!FileExists(Cfg))
+                    {
+                        fprintf(stderr, "Cannot find '%s' in:\n"
+                                "\t~/.fte,\n""\t%slocalconfig,\n"
+                                "\t%sconfig, or\n"
+                                "\t.",
+                                CfgName, StartDir, StartDir);
+                    }
+                }
+            }
+        }
+#else // UNIX
         SlashDir(last);
         strcat(last, CfgName);
         ExpandPath(last, Cfg);
+#endif // UNIX
     }
     // puts(Cfg);
 
@@ -1692,13 +1938,17 @@ int LoadFile(const char *WhereName, const char *CfgName, int Level) {
         fprintf(stderr, "Cannot stat '%s', errno=%d\n", Cfg, errno);
         return -1;
     }
-    buffer = (char *) malloc(statbuf.st_size);
-    if (buffer == 0) {
+    buffer = (char *) malloc(statbuf.st_size+1);
+    if (buffer == NULL) {
         close(fd);
         return -1;
     }
+
+    buffer[statbuf.st_size] = 0; // add null to end of buffer, NOTE: allocated statbuf.st_size + 1
+
     if (read(fd, buffer, statbuf.st_size) != statbuf.st_size) {
         close(fd);
+        free(buffer);
         return -1;
     }
     close(fd);
@@ -1709,6 +1959,21 @@ int LoadFile(const char *WhereName, const char *CfgName, int Level) {
     cp.line = 1;
     cp.name = Cfg;
 
+    // preprocess configuration file
+    rc = PreprocessConfigFile(cp);
+    if (rc == -1) {
+        Fail(cp, "Preprocess failed");
+    }
+
+    if (preprocess_only == true) {
+	printf("%s", cp.a);
+    }
+
+    // reset pointers
+    cp.a = cp.c = buffer;
+    cp.z = cp.a + cp.sz;
+    cp.line = 1;
+
     rc = ParseConfigFile(cp);
     // puts("End Loading file");
     if (Level == 0)
