@@ -14,6 +14,7 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <process.h>
+#include "sysdep.h"
 
 #define SEM_PREFIX  "\\SEM32\\PMCLIPS\\"
 #define MEM_PREFIX  "\\SHAREMEM\\PMCLIPS\\"
@@ -30,7 +31,7 @@ HEV hevPut;
 HEV hevEnd;
 HMUX hmuxWait;
 
-void clipsrv(void *foo) {
+void _LNK_CONV clipsrv(void *foo) {
     HAB hab;
     HMQ hmq;
     
@@ -169,7 +170,7 @@ int main() {
     hab = WinInitialize(0);
     
     hmq = WinCreateMsgQueue(hab, 0);
-#if defined(EMX) || defined(__TOS_OS2__)
+#if defined(__EMX__) || defined(__TOS_OS2__)
     _beginthread(clipsrv, NULL, 8192, 0);
 #else
     _beginthread(clipsrv, 8192, 0);
