@@ -47,7 +47,26 @@ extern char FileBuffer[RWBUFSIZE];
 
 #define NextTab(pos,ts) (((pos) / (ts) + 1) * (ts))
 
-#define GapLine(x,g,c,a) (((x) < (g)) ? (x) : ((x) + (a) - (c)))
+// x before gap -> x
+// x less than count -> after gap
+// count - 1 before gap -> count - 1
+// after gap -> allocated - 1
+//#define GapLine(x,g,c,a) (((x) < (g)) ? (x) : (x) < (c) ? ((x) + (a) - (c)) : (c) - 1 < (g) ? (c) - 1 : (a) - 1 )
+// Use inline to make it easier to read/debug
+int inline GapLine(int No, int Gap, int Count, int Allocated)
+{
+    int rc = -1;
+    if (No < Gap)
+        rc = No;
+    else if (No < Count)
+        rc = No + Allocated - Count;
+    else if (Count - 1 < Gap)
+        rc = Count - 1;
+    else
+        rc = Allocated - 1;
+    return rc;
+}
+
 
 typedef class ELine* PELine;
 typedef class EPoint* PEPoint;
@@ -75,7 +94,7 @@ public:
     int Row;
     int Col;
 
-    EPoint(EPoint& M) { Row = M.Row; Col = M.Col; }
+//    EPoint(EPoint &M) { Row = M.Row; Col = M.Col; }
     EPoint(int aRow = 0, int aCol = 0) { Row = aRow; Col = aCol; }
     ~EPoint() {}
 };
@@ -98,7 +117,7 @@ typedef struct _RoutineList {
 #endif
 
 #ifdef CONFIG_BOOKMARKS
-typedef struct {
+typedef struct _EBookmark {
     char *Name;
     EPoint BM;
 } EBookmark;
@@ -318,7 +337,7 @@ public:
     int SaveRegion(EPoint *A, EPoint *Z, int FH, int AddCR, int AddLF, int Mode);
     
     int AssertLine(int Line);
-    int InsertLine(EPoint Pos, int ACount, char *AChars);
+    int InsertLine(EPoint Pos, int ACount, const char *AChars);
     
     int UpdateMarker(int Type, int Line, int Col, int Lines, int Cols);
     int UpdateMark(EPoint &M, int Type, int Line, int Col, int Lines, int Cols);
@@ -384,10 +403,10 @@ public:
     int UnTabPoint(int Row, int Col);
     int InsLine(int Row, int DoAppend, int DoMark = 1);
     int DelChars(int Row, int Ofs, int ACount);
-    int InsChars(int Row, int Ofs, int ACount, char *Buffer);
-    int ChgChars(int Row, int Ofs, int ACount, char *Buffer);
+    int InsChars(int Row, int Ofs, int ACount, const char *Buffer);
+    int ChgChars(int Row, int Ofs, int ACount, const char *Buffer);
     int DelText(int Row, int Col, int ACount, int DoMark = 1);
-    int InsText(int Row, int Col, int ACount, char *Buffer, int DoMark = 1);
+    int InsText(int Row, int Col, int ACount, const char *Buffer, int DoMark = 1);
     int InsLineText(int Row, int Col, int ACount, int Pos, PELine Line);
     int SplitLine(int Row, int Col);
     int JoinLine(int Row, int Col);
@@ -414,7 +433,8 @@ public:
 // Utility Routines
 /////////////////////////////////////////////////////////////////////////////
     
-    int LineIndented(int Row);
+    int LineIndented(int Row, const char *indentchars = NULL);
+    int LineIndentedCharCount(ELine *l, const char *indentchars);
     int IndentLine(int Row, int Indent);
 #ifdef CONFIG_SYNTAX_HILIT
     int GetMap(int Row, int *StateLen, hsState **StateMap);
@@ -438,6 +458,7 @@ public:
     int RemoveBookmark(char *Name);
     int GetBookmark(char *Name, EPoint &P);
     int GotoBookmark(char *Name);
+    int GetBookmarkForLine(int searchFrom, int searchForLine, char *&Name, EPoint &P);
 #endif
     
 /////////////////////////////////////////////////////////////////////////////
@@ -574,7 +595,7 @@ public:
 
     int     InsertChar(char aCh);
     int     TypeChar(char aCh);
-    int     InsertString(char *aStr, int aCount);
+    int     InsertString(const char *aStr, int aCount);
     int     InsertSpacesToTab(int TSize);
     int     InsertTab();
     int     InsertSpace();
@@ -604,8 +625,8 @@ public:
     int     BlockMarkStream();
     int     BlockMarkLine();
     int     BlockMarkColumn();
-    int     BlockReadFrom(char *aFileName, int blockMode);
-    int     BlockWriteTo(char *aFileName, int Append = 0);
+    int     BlockReadFrom(const char *aFileName, int blockMode);
+    int     BlockWriteTo(const char *aFileName, int Append = 0);
     int     BlockExtendBegin();
     int     BlockExtendEnd();
     int     BlockReIndent();
@@ -613,6 +634,7 @@ public:
     int     BlockPasteStream();
     int     BlockPasteLine();
     int     BlockPasteColumn();
+    int     BlockPasteOver();
     int     BlockSelectWord();
     int     BlockSelectLine();
     int     BlockSelectPara();
@@ -638,7 +660,9 @@ public:
     int     ToggleWordWrap();
     int     ToggleTrim();
     int     ToggleShowMarkers();
-    
+    int     ToggleHilitTags ();
+    int     ToggleShowBookmarks ();
+
     int     SetLeftMargin();
     int     SetRightMargin();
     
@@ -696,6 +720,10 @@ public:
     int MoveToLine(ExState &State);
     int FoldCreateByRegexp(ExState &State);
 #ifdef CONFIG_BOOKMARKS
+    int PlaceUserBookmark(const char *n,EPoint P);
+    int RemoveUserBookmark(const char *n);
+    int GotoUserBookmark(const char *n);
+    int GetUserBookmarkForLine(int searchFrom, int searchForLine, char *&Name, EPoint &P);
     int PlaceBookmark(ExState &State);
     int RemoveBookmark(ExState &State);
     int GotoBookmark(ExState &State);
@@ -758,7 +786,7 @@ public:
     int FoldCreateAtRoutines();
 };
 
-extern EBuffer *SS;
+extern EBuffer *SSBuffer;
 extern SearchReplaceOptions LSearch;
 
 extern int suspendLoads;
