@@ -45,22 +45,22 @@ void EFrame::UpdateMenu() {
         Map = V->GetEventMap();
 
     if (Map != CMap || CMap == 0) {
-        const char *Menu = 0;
+        const char *NMenu = 0;
         const char *OMenu = 0;
         // set menu
 
         if (CMap)
             OMenu = CMap->GetMenu(EM_MainMenu);
         if (Map)
-            Menu = Map->GetMenu(EM_MainMenu);
-        if (Menu == 0)
-            Menu = "Main";
+            NMenu = Map->GetMenu(EM_MainMenu);
+        if (NMenu == 0)
+            NMenu = "Main";
         CMap = Map;
 
-        if (OMenu && strcmp(OMenu, Menu) == 0) {
+        if (OMenu && strcmp(OMenu, NMenu) == 0) {
             // ok
         } else {
-            SetMenu(Menu);
+            SetMenu(NMenu);
         }
     } /*else if (CMap == 0 && Map == 0) {
         SetMenu("Main");
@@ -437,7 +437,7 @@ int EGUI::WinHSplit(GxView *View) {
     return 1;
 }
 
-int EGUI::WinClose(GxView *V) {
+int EGUI::WinClose(GxView * /*V*/) {
     EView *View = ActiveView;
 
     if (View->Next == View) {
@@ -513,6 +513,12 @@ int EGUI::ExitEditor(EView *View) {
 #endif
 
     while (ActiveModel) {
+        if (View->Model->GetContext() == CONTEXT_ROUTINES)  // Never delete Routine models directly
+        {
+            ActiveModel = ActiveModel->Next;
+            View->SelectModel(ActiveModel);
+        }
+
         View->Model->DeleteRelated();  // delete related views first
 
         while (View->Model->Next != View->Model &&
@@ -603,7 +609,7 @@ int EGUI::FrameNew() {
     if (!multiFrame() && frames)
         return 0;
 
-    new EFrame(ScreenSizeX, ScreenSizeY);
+    (void)new EFrame(ScreenSizeX, ScreenSizeY);
     assert(frames != 0);
 
     //frames->SetMenu("Main"); //??
@@ -611,7 +617,7 @@ int EGUI::FrameNew() {
     view = new GxView(frames);
     assert(view != 0);
 
-    new EView(ActiveModel);
+    (void)new EView(ActiveModel);
     assert(ActiveView != 0);
 
     edit = new ExModelView(ActiveView);
@@ -635,7 +641,7 @@ int EGUI::FrameClose(GxView *View) {
     return 1;
 }
 
-int EGUI::FrameNext(GxView *View) {
+int EGUI::FrameNext(GxView * /*View*/) {
     if (!frames->isLastFrame()) {
         frames->Next->Activate();
         return 1;
@@ -643,7 +649,7 @@ int EGUI::FrameNext(GxView *View) {
     return 0;
 }
 
-int EGUI::FramePrev(GxView *View) {
+int EGUI::FramePrev(GxView * /*View*/) {
     if (!frames->isLastFrame()) {
         frames->Prev->Activate();
         return 1;
@@ -735,20 +741,20 @@ void EGUI::DoLoadDesktopOnEntry(int &/*argc*/, char **argv) {
 #endif
 
 void EGUI::EditorInit() {
-    SS = new EBuffer(0, (EModel **)&SS, "Scrap");
-    assert(SS != 0);
-    BFI(SS, BFI_Undo) = 0; // disable undo for clipboard
+    SSBuffer = new EBuffer(0, (EModel **)&SSBuffer, "Scrap");
+    assert(SSBuffer != 0);
+    BFI(SSBuffer, BFI_Undo) = 0; // disable undo for clipboard
     ActiveModel = 0;
 }
 
-int EGUI::InterfaceInit(int &argc, char **argv) {
+int EGUI::InterfaceInit(int &/*argc*/, char ** /*argv*/) {
     if (FrameNew() == 0)
         DieError(1, "Failed to create window\n");
     return 0;
 }
 
 #ifdef CONFIG_HISTORY
-void EGUI::DoLoadHistoryOnEntry(int &argc, char **argv) {
+void EGUI::DoLoadHistoryOnEntry(int &/*argc*/, char **argv) {
     if (HistoryFileName[0] == 0) {
 #ifdef UNIX
         ExpandPath("~/.fte-history", HistoryFileName);
@@ -774,6 +780,9 @@ void EGUI::DoLoadHistoryOnEntry(int &argc, char **argv) {
 void EGUI::DoSaveHistoryOnExit() {
     if (KeepHistory && HistoryFileName[0] != 0)
         SaveHistory(HistoryFileName);
+
+    // since we are exiting, free history
+    ClearHistory();
 }
 #endif
 
@@ -791,7 +800,8 @@ int EGUI::CmdLoadFiles(int &argc, char **argv) {
     for (int Arg = 1; Arg < argc; Arg++) {
         if (!QuoteAll && !QuoteNext && (argv[Arg][0] == '-')) {
             if (argv[Arg][1] == '-') {
-                QuoteAll = 1;
+                if (strncmp(argv[Arg], "--debug", 7) != 0)
+                    QuoteAll = 1;
             } else if (argv[Arg][1] == '!') {
                 // handled before
             } else if (argv[Arg][1] == 'c' || argv[Arg][1] == 'C') {
@@ -921,32 +931,39 @@ int EGUI::Start(int &argc, char **argv) {
 }
 
 void EGUI::EditorCleanup() {
-    if (ActiveModel) {
+    if (ActiveModel != NULL) {
         EModel *B, *N, *A;
-
-        B = A = ActiveModel;
-        while (B != A) {
+        
+       	B = A = ActiveModel;
+        do {
             N = B->Next;
             delete B;
             B = N;
-        }
+        } while (B != A);
     }
-    ActiveModel = 0;
+    ActiveModel = NULL;
 
-    delete SS;
-    SS = 0;
+    delete SSBuffer;
+    SSBuffer = NULL;
 
-    if (ActiveView) {
-        EView *BW, *NW, *AW;
+    if (ActiveView != NULL) {
+       	EView *BW, *NW;
 
-        BW = AW = ActiveView;
-        while (BW != AW) {
-            NW = BW->Next;
-            delete BW;
-            BW = NW;
-        }
+       	// If EView what is about to be deleted is currently ActiveView, ActiveView moves to next one
+       	// or if there is no next, it will be set as NULL.
+       	while ((BW = ActiveView) != NULL) {
+       		NW = BW->Next;
+       		delete BW;
+       	}
+        //EView *BW, *NW, *AW;
+        //BW = AW = ActiveView;
+        //do {
+        //    NW = BW->Next;
+        //    delete BW;
+        //    BW = NW;
+        //} while (BW != AW);
     }
-    ActiveView = 0;
+    ActiveView = NULL;
 }
 
 void EGUI::InterfaceCleanup() {
@@ -959,6 +976,111 @@ void EGUI::Stop() {
     DoSaveHistoryOnExit();
 #endif
 
+    // free macros
+    if (Macros!=NULL)
+    {
+        int i;
+
+        while (CMacros--)
+        {
+            free(Macros[CMacros].Name);
+
+            for (i=0; i<Macros[CMacros].Count; i++)
+            {
+                if (Macros[CMacros].cmds[i].type == CT_STRING)
+                {
+                    free(Macros[CMacros].cmds[i].u.string);
+                }
+            }
+
+            free(Macros[CMacros].cmds);
+        }
+
+        free(Macros);
+
+        Macros = NULL;
+    }
+
+    // free colorizers
+    {
+        EColorize *p;
+
+        while ((p = Colorizers) != NULL) {
+            Colorizers = Colorizers->Next;
+            delete p;
+        }
+    }
+
+    // free event maps
+    {
+        EEventMap *em;
+                  
+        while ((em = EventMaps) != NULL) {
+            EventMaps = EventMaps->Next;
+            delete em;
+        }
+    }
+
+    // free modes
+    {
+        EMode *m;
+                  
+        while ((m = Modes) != NULL) {
+            Modes = Modes->fNext;
+            delete m;
+        }
+    }
+
+    // free menus
+    if (Menus)
+    {
+        int mc, c;
+
+        while(MenuCount--)
+        {
+            mc = MenuCount;
+
+            free(Menus[mc].Name);
+
+            while(Menus[mc].Count--)
+            {
+                c = Menus[mc].Count;
+                free(Menus[mc].Items[c].Name);
+            }
+
+            free(Menus[MenuCount].Items);
+        }
+
+        free(Menus);
+
+        Menus = NULL;
+    }
+
+    // free completion rexexp filter
+    {
+        extern RxNode *CompletionFilter;
+
+        RxFree(CompletionFilter);
+    }
+
+    // free CRegexp array from o_messages.cpp
+    {
+        FreeCRegexp();
+    }
+
+#ifdef CONFIG_OBJ_CVS
+    // free CvsIgnoreRegexp array from o_messages.cpp
+    {
+        FreeCvsIgnoreRegexp();
+    }
+#endif
+
+    // free configuration file path
+    {
+        free(ConfigSourcePath);
+        ConfigSourcePath = NULL;
+    }
+
     EditorCleanup();
 
     InterfaceCleanup();
