@@ -14,6 +14,9 @@
 #include "feature.h"
 #include "sysdep.h"
 
+#ifdef NEED_LOG_H
+#include "log.h"
+#endif // NEED_LOG_H
 #include "console.h"
 #include "gui.h"
 #include "c_desktop.h"
@@ -48,7 +51,18 @@
 #include "o_buflist.h"
 #include "o_routine.h"
 #include "o_modemap.h"
+#include "e_cvslog.h"
+#include "o_cvsbase.h"
+#include "o_cvs.h"
+#include "o_cvsdiff.h"
 #include "c_commands.h"
 #include "c_history.h"
 
+#if defined(_DEBUG) && defined(MSVC) && defined(MSVCDEBUG)
+#include <crtdbg.h>
+
+#define new new( _CLIENT_BLOCK, __FILE__, __LINE__)
+
+#endif //_DEBUG && MSVC && MSVCDEBUG
+
 #endif
