@@ -89,26 +89,47 @@
 
 #define hsPerl_HereDoc     44 // hack (eod not detected properly)
 
-#define opQ  1
+enum PerlOps
+{
+    opQ,
+    opQQ,
+    opQW,
+    opQX,
+    opQR,
+    opM,
+    opS,
+    opTR
+};
+/*#define opQ  1
 #define opQQ 2
 #define opQW 3
 #define opQX 4
 #define opM  5
 #define opS  6
-#define opTR 7
+#define opTR 7*/
 
 int Hilit_PERL(EBuffer *BF, int /*LN*/, PCell B, int Pos, int Width, ELine *Line, hlState &State, hsState *StateMap, int *ECol) {
-    ChColor *Colors = BF->Mode->fColorize->Colors;
     int j;
-    HILIT_VARS(Colors[CLR_Normal], Line);
+    HILIT_VARS(BF->Mode->fColorize->Colors, Line);
     int firstnw = 0;
     int op;
     int setHereDoc = 0;
+    int inSub = 0;
+#define MAXSEOF 100
+    static char hereDocKey[MAXSEOF];
 
     C = 0;
     NC = 0;
+    int isEOHereDoc = 0;
     for(i = 0; i < Line->Count;) {
         if (*p != ' ' && *p != 9) firstnw++;
+        if ((State & X_MASK) == hsPerl_HereDoc && 0 == i)
+        {
+            isEOHereDoc = strlen(hereDocKey) == (size_t)len &&
+                strncmp(hereDocKey, Line->Chars, len) == 0;
+        }
+        if (*p == '{' && inSub)
+            inSub = 0;
         IF_TAB() else {
             //         printf("State = %d pos = %d", State, i); fflush(stdout);
             switch (State & X_MASK) {
@@ -124,7 +145,7 @@ int Hilit_PERL(EBuffer *BF, int /*LN*/, PCell B, int Pos, int Width, ELine *Line
                     p[6] == '_')
                 {
                     State = hsPerl_Data;
-                    Color = Colors[CLR_Comment];
+                    Color = CLR_Comment;
                 hilit5:
                     ColorNext();
                 //hilit4:
@@ -136,11 +157,19 @@ int Hilit_PERL(EBuffer *BF, int /*LN*/, PCell B, int Pos, int Width, ELine *Line
                 hilit:
                     ColorNext();
                     continue;
-                } else if (i == 0 && X_NOT(State) && (*p == '=') && len > 4 &&
-                    p[1] == 'h' && p[2] == 'e' && p[3] == 'a' && p[4] == 'd')
+                } else if (
+                           i == 0 && X_NOT(State) && (*p == '=') && len > 4 &&
+                           (
+                            strncmp(p+1, "head", 4) == 0 ||
+                            strncmp(p+1, "item", 4) == 0 ||
+                            strncmp(p+1, "over", 4) == 0 ||
+                            strncmp(p+1, "back", 4) == 0 ||
+                            strncmp(p+1, "pod",  3) == 0
+                           )
+                          )
                 {
                     State = hsPerl_Docs;
-                    Color = Colors[CLR_Comment];
+                    Color = CLR_Comment;
                     goto hilit5;
                 } else if (isalpha(*p) || *p == '_') {
                     op = -1;
@@ -153,15 +182,19 @@ int Hilit_PERL(EBuffer *BF, int /*LN*/, PCell B, int Pos, int Width, ELine *Line
                     if (BF->GetHilitWord(j, &Line->Chars[i], Color)) {
                         //Color = hcPERL_Keyword;
                         State = hsPerl_Keyword;
+                        if (strncmp(p, "sub", 3) == 0)
+                        {
+                            inSub = 1;
+                        }
                     } else {
                         int x;
                         x = i + j;
                         while ((x < Line->Count) && 
                                ((Line->Chars[x] == ' ') || (Line->Chars[x] == 9))) x++;
                         if ((x < Line->Count) && (Line->Chars[x] == '(')) {
-                            Color = Colors[CLR_Function];
+                            Color = CLR_Function;
                         } else {
-                            Color = Colors[CLR_Normal];
+                            Color = CLR_Normal;
                         }
                         State = hsPerl_Normal;
                     }
@@ -169,17 +202,22 @@ int Hilit_PERL(EBuffer *BF, int /*LN*/, PCell B, int Pos, int Width, ELine *Line
                         if (*p == 'q') op = opQ;
                         else if (*p == 's' || *p == 'y') op = opS;
                         else if (*p == 'm') op = opM;
-                    } else if (j == 2) {
+                    } else if (j >= 2) {
                         if (*p == 'q') {
-                            if (*p == 'q') op = opQQ;
-                            else if (*p == 'w') op = opQW;
-                            else if (*p == 'x') op = opQX;
+                            if (p[1] == 'q') op = opQQ;
+                            else if (p[1] == 'w') op = opQW;
+                            else if (p[1] == 'r') op = opQR;
+                            else if (p[1] == 'x') op = opQX;
                         } else if (*p == 't' && p[1] == 'r') op = opTR;
+                        if (op != -1 && j > 2 && p[2] == '\'')
+			    j = 2;
+			else if (op != -1 && kwd(p[2]))
+                            op = -1;
                     }
                     if (StateMap)
                         memset(StateMap + i, State, j);
                     if (B) 
-                        MoveMem(B, C - Pos, Width, Line->Chars + i, Color, j);
+                        MoveMem(B, C - Pos, Width, Line->Chars + i, HILIT_CLRD(), j);
                     i += j;
                     len -= j;
                     p += j;
@@ -188,29 +226,30 @@ int Hilit_PERL(EBuffer *BF, int /*LN*/, PCell B, int Pos, int Width, ELine *Line
                     switch (op) {
                     case opQ:
                         State = hsPerl_Quote1Op;   // q{} operator
-                        Color = Colors[CLR_Punctuation];
+                        Color = CLR_Punctuation;
                         continue;
                         
                     case opQQ:
                     case opQW:
                     case opQX:
                         State = hsPerl_Quote1Op;   // qq{} qx{} qw{} operators
-                        Color = Colors[CLR_Punctuation];
+                        Color = CLR_Punctuation;
                         continue;
                         
                     case opM:
-                        State = hsPerl_Regexp1Op;   // m{} operator
-                        Color = Colors[CLR_Punctuation];
+                    case opQR:
+                        State = hsPerl_Regexp1Op;   // m{} qr{} operator
+                        Color = CLR_Punctuation;
                         continue;
                         
                     case opTR:
                         State = hsPerl_Regexp2Op;   // tr{} operators
-                        Color = Colors[CLR_RegexpDelim];
+                        Color = CLR_RegexpDelim;
                         continue;
                         
                     case opS:
                         State = hsPerl_Regexp2Op;   // s{}{} operator
-                        Color = Colors[CLR_Punctuation];
+                        Color = CLR_Punctuation;
                         continue;
                         
                     default:
@@ -220,21 +259,30 @@ int Hilit_PERL(EBuffer *BF, int /*LN*/, PCell B, int Pos, int Width, ELine *Line
                 } else if (len >= 2 && ((*p == '-' && p[1] == '-') || (*p == '+' && p[1] == '+'))) {
                     hlState s = State;
                     State = hsPerl_Punct;
-                    Color = Colors[CLR_Punctuation];
+                    Color = CLR_Punctuation;
                     ColorNext();
                     ColorNext();
                     State = s;
                     continue;
+                } else if (inSub && *p == '(') {
+                    State = hsPerl_Punct;
+                    Color = CLR_Punctuation;
+                    ColorNext();
+                    Color = CLR_Variable;
+                    while (*p != ')' && len)
+                        ColorNext();
+                    State = hsPerl_Normal;
+                    continue;
                 } else if (len >= 2 && *p == '&' && (p[1] == '&' || isspace(p[1]))) {
                     State = hsPerl_Punct;
-                    Color = Colors[CLR_Punctuation];
+                    Color = CLR_Punctuation;
                     ColorNext();
                     ColorNext();
                     State = hsPerl_Normal;
                     continue;
                 } else if (*p == '&' && (len < 2 || p[1] != '&') && X_NOT(State)) {
                     State = hsPerl_Function;
-                    Color = Colors[CLR_Function];
+                    Color = CLR_Function;
                     ColorNext();
                     while ((len > 0) && (*p == '$' ||
                                          *p == '@' ||
@@ -251,36 +299,61 @@ int Hilit_PERL(EBuffer *BF, int /*LN*/, PCell B, int Pos, int Width, ELine *Line
                 } else if ((*p == '$') && (len > 1) &&
                            ((p[1] == '$') || p[1] == '"')) {
                     State = hsPerl_Variable;
-                    Color = Colors[CLR_Variable];
+                    Color = CLR_Variable;
                     ColorNext();
                     ColorNext();
                     State = hsPerl_Normal | X_BIT;
                     continue;
                 } else if (*p == '$' || *p == '@' || *p == '\\' || (len > 2 && (*p == '%' || *p == '*') && X_NOT(State))) {
+                    char var_type = *p;
                     State = hsPerl_Variable;
-                    Color = Colors[CLR_Variable];
+                    Color = CLR_Variable;
                     ColorNext();
                     while ((len > 0) && ((*p == ' ') || (*p == '\t'))) {
                         IF_TAB() else
                             ColorNext();
                     }
-                    while ((len > 0) && (*p == '$' ||
+                    /*while ((len > 0) && (*p == '$' ||
                                          *p == '@' ||
                                          *p == '*' ||
                                          *p == '%' ||
                                          *p == '\\'))
+                        ColorNext();*/
+                    char first = *p;
+                    if (len > 0 && *p != ' ' && *p != '\t' && *p != '"')
                         ColorNext();
-                    if (len > 0 && *p != '{' && *p != ' ' && *p != '\t' && *p != '"' && *p != '\'')
-                        ColorNext();
-                    while ((len > 0) && (isalnum(*p) || 
-                                         *p == '_' || 
-                                         *p == '\''))
-                        ColorNext();
+                    // the following are one-character-ONLY
+                    if (
+                        (var_type == '$' && strchr("_&`'+*.!/|,\\\";#%=-~:?$<>()[]", first) != NULL) ||
+                        (var_type == '@' && strchr("_", first) != NULL)
+                       )
+                    {
+                        // nothing.
+                    }
+                    // the following are one-or-two-characters-ONLY
+                    else if (first == '^')
+                    {
+                        if (len > 0 && isalpha(*p))
+                            ColorNext();
+                    }
+                    else if (first == '{')
+                    {
+                        UntilMatchBrace(first, ColorNext());
+                    }
+                    else
+                    {
+                        while ((len > 0) && (isalnum(*p) ||
+                                             (first == '{' && (*p == '^' || *p == '}')) ||
+                                             *p == '_' ||
+                                             *p == '\'')
+                              )
+                            ColorNext();
+                    }
                     State = hsPerl_Normal | X_BIT;
                     continue;
                 } else if ((len >= 2) && (*p == '0') && (*(p+1) == 'x')) {
                     State = hsPerl_Number;
-                    Color = Colors[CLR_Number];
+                    Color = CLR_Number;
                     ColorNext();
                     ColorNext();
                     while (len && (isxdigit(*p) || *p == '_')) ColorNext();
@@ -290,7 +363,7 @@ int Hilit_PERL(EBuffer *BF, int /*LN*/, PCell B, int Pos, int Width, ELine *Line
                     continue;
                 } else if (isdigit(*p)) {
                     State = hsPerl_Number;
-                    Color = Colors[CLR_Number];
+                    Color = CLR_Number;
                     ColorNext();
                     while (len && (isdigit(*p) || (*p == 'e' || *p == 'E' || *p == '_'))) ColorNext();
                     //                    if (len && (toupper(*p) == 'U')) ColorNext();
@@ -299,81 +372,117 @@ int Hilit_PERL(EBuffer *BF, int /*LN*/, PCell B, int Pos, int Width, ELine *Line
                     continue;
                 } else if (*p == '\'') {
                     State = QSET(hsPerl_String1, '\'');
-                    Color = Colors[CLR_String];
+                    Color = CLR_String;
                     goto hilit;
                 } else if (*p == '"') {
                     State = QSET(hsPerl_String2, '"');
-                    Color = Colors[CLR_String];
+                    Color = CLR_String;
                     goto hilit;
                 } else if (*p == '<' && len > 2 && p[1] == '<' &&
-                           (p[2] == '"' || p[2] == '\'' || p[2] == '_' || (p[2] >= 'A' && p[2] <= 'Z')))
+                           (p[2] == '"' || p[2] == '\'' || p[2] == '_' || (toupper(p[2]) >= 'A' && toupper(p[2]) <= 'Z')))
                 {
+                    int hereDocKeyLen;
+                    int offset = 2;
+                    if (p[2] == '"' || p[2] == '\'')
+                        offset++;
                     setHereDoc++;
+                    for (hereDocKeyLen = 0;
+                         hereDocKeyLen < len && (
+                                                 p[offset + hereDocKeyLen] == '_' ||
+                                                 (toupper(p[offset + hereDocKeyLen]) >= 'A' && toupper(p[offset + hereDocKeyLen]) <= 'Z')
+                                                );
+                         ++hereDocKeyLen)
+                    {
+                        hereDocKey[hereDocKeyLen] = p[offset + hereDocKeyLen];
+                    }
+                    hereDocKey[hereDocKeyLen] = '\0';
                     State = hsPerl_Punct;
-                    Color = Colors[CLR_Punctuation];
+                    Color = CLR_Punctuation;
                     ColorNext();
                     State = hsPerl_Normal;
                     continue;
                 } else if (*p == '`') {
                     State = QSET(hsPerl_StringBk, '`');
-                    Color = Colors[CLR_String];
+                    Color = CLR_String;
                     goto hilit;
                 } else if (*p == '#') {
                     State = hsPerl_Comment | (State & X_BIT);
                     continue;
                 } else if (X_NOT(State) && *p == '/') {
                     State = QSET(hsPerl_Regexp1, '/');
-                    Color = Colors[CLR_RegexpDelim];
+                    Color = CLR_RegexpDelim;
                     goto hilit;
-                } else if (X_NOT(State) && *p == '-' && (len >= 2 && isalpha(p[1]))) {
-                    Color = Colors[CLR_Punctuation];
+                } else if (X_NOT(State) &&
+                           *p == '-' &&
+                           len >= 2 &&
+                           isalpha(p[1])
+                          ) {
+                    Color = CLR_Normal; // default.
+                    if (strchr("wrxoRWXOezsfdlpSbctugkTB", p[1]) != NULL) {
+                        Color = CLR_Punctuation; // new default.
+                        if (len > 2) {
+                            switch(p[2]) {
+                            case '_': // there may be others...
+                                Color = CLR_Normal;
+                                break;
+                            default:
+                                if (isalnum(p[2]))
+                                    Color = CLR_Normal;
+                                break;
+                            }
+                        }
+                    }
                     ColorNext();
                     ColorNext();
                     State = hsPerl_Normal;
                     continue;
                 } else if (*p == ')' || *p == ']') {
                     State = hsPerl_Punct;
-                    Color = Colors[CLR_Punctuation];
+                    Color = CLR_Punctuation;
                     ColorNext();
                     State = hsPerl_Normal | X_BIT;
                     continue;
                 } else if (ispunct(*p)) {
                     State = hsPerl_Punct;
-                    Color = Colors[CLR_Punctuation];
+                    Color = CLR_Punctuation;
                     ColorNext();
                     State = hsPerl_Normal;
                     continue;
                 }
-                Color = Colors[CLR_Normal];
+                Color = CLR_Normal;
                 goto hilit;
             case hsPerl_Quote1Op:
                 if (*p != ' ' && !kwd(*p)) {
                     if (IS_OBRACE(*p))
+                    {
                         State = QSET(hsPerl_Quote1M,
                                      (1U << 2) | NUM_BRACE(*p));
+                    }
                     else
+                    {
                         State = QSET(hsPerl_Quote1, *p);
-                    Color = Colors[CLR_QuoteDelim];
+                    }
+                    Color = CLR_QuoteDelim;
                     goto hilit;
                 } else if (kwd(*p)) {
                     State = hsPerl_Normal | X_BIT;
                     continue;
                 }
-                Color = Colors[CLR_Punctuation];
+                Color = CLR_Punctuation;
                 goto hilit;
             case hsPerl_Quote1:
-                Color = Colors[CLR_String];
+                Color = CLR_String;
                 if ((len >= 2) && (*p == '\\')) {
                     goto hilit2;
                 } else if (*p == QCHAR(State)) {
-                    Color = Colors[CLR_QuoteDelim];
+                    Color = CLR_QuoteDelim;
                     ColorNext();
                     State = hsPerl_Normal | X_BIT;
                     continue;
                 }
                 goto hilit;
             case hsPerl_Quote1M:
-                Color = Colors[CLR_String];
+                Color = CLR_String;
                 if ((len >= 2) && (*p == '\\')) {
                     goto hilit2;
                 } else if (GET_BRACE(QCHAR(State) & 3) == *p) {
@@ -382,7 +491,7 @@ int Hilit_PERL(EBuffer *BF, int /*LN*/, PCell B, int Pos, int Width, ELine *Line
                 } else if (IS_MBRACE(GET_BRACE(QCHAR(State) & 3), *p)) {
                     State -= 1 << (2 + 8);
                     if ((QCHAR(State) >> 2) == 0) {
-                        Color = Colors[CLR_QuoteDelim];
+                        Color = CLR_QuoteDelim;
                         ColorNext();
                         State = hsPerl_Normal | X_BIT;
                     } else
@@ -391,28 +500,28 @@ int Hilit_PERL(EBuffer *BF, int /*LN*/, PCell B, int Pos, int Width, ELine *Line
                 }
                 goto hilit;
             case hsPerl_Regexp1Op:
-                if (*p != ' ' && !kwd(*p)) {
+                if (*p != ' ') { // && !kwd(*p)) {
                     if (IS_OBRACE(*p))
                         State = QSET(hsPerl_Regexp1M,
                                      (1U << 2) | NUM_BRACE(*p));
                     else
                         State = QSET(hsPerl_Regexp1, *p);
-                    Color = Colors[CLR_RegexpDelim];
+                    Color = CLR_RegexpDelim;
                     goto hilit;
                 } else if (kwd(*p)) {
                     State = hsPerl_Normal | X_BIT;
                     continue;
                 }
-                Color = Colors[CLR_Regexp];
+                Color = CLR_Regexp;
                 goto hilit;
             case hsPerl_Regexp1:
-                Color = Colors[CLR_Regexp];
+                Color = CLR_Regexp;
                 if ((len >= 2) && (*p == '\\')) {
                     goto hilit2;
                 } else if (*p == QCHAR(State)) {
-                    Color = Colors[CLR_RegexpDelim];
+                    Color = CLR_RegexpDelim;
                     ColorNext();
-                    Color = Colors[CLR_Punctuation];
+                    Color = CLR_Punctuation;
                     while (len > 0 && isalpha(*p))
                         ColorNext();
                     State = hsPerl_Normal | X_BIT;
@@ -420,7 +529,7 @@ int Hilit_PERL(EBuffer *BF, int /*LN*/, PCell B, int Pos, int Width, ELine *Line
                 }
                 goto hilit;
             case hsPerl_Regexp1M:
-                Color = Colors[CLR_Regexp];
+                Color = CLR_Regexp;
                 if ((len >= 2) && (*p == '\\')) {
                     goto hilit2;
                 } else if (GET_BRACE(QCHAR(State) & 3) == *p) {
@@ -429,9 +538,9 @@ int Hilit_PERL(EBuffer *BF, int /*LN*/, PCell B, int Pos, int Width, ELine *Line
                 } else if (IS_MBRACE(GET_BRACE(QCHAR(State) & 3), *p)) {
                     State -= 1 << (2 + 8);
                     if ((QCHAR(State) >> 2) == 0) {
-                        Color = Colors[CLR_RegexpDelim];
+                        Color = CLR_RegexpDelim;
                         ColorNext();
-                        Color = Colors[CLR_Punctuation];
+                        Color = CLR_Punctuation;
                         while (len > 0 && isalpha(*p))
                             ColorNext();
                         State = hsPerl_Normal | X_BIT;
@@ -441,26 +550,26 @@ int Hilit_PERL(EBuffer *BF, int /*LN*/, PCell B, int Pos, int Width, ELine *Line
                 }
                 goto hilit;
             case hsPerl_Regexp2Op:
-                if (*p != ' ' && !kwd(*p)) {
+                if (*p != ' ') { // && !kwd(*p)) {
                     if (IS_OBRACE(*p))
                         State = QSET(hsPerl_Regexp2M,
                                      (1U << 2) | NUM_BRACE(*p));
                     else
                         State = QSET(hsPerl_Regexp2, *p);
-                    Color = Colors[CLR_RegexpDelim];
+                    Color = CLR_RegexpDelim;
                     goto hilit;
                 } else if (kwd(*p)) {
                     State = hsPerl_Normal | X_BIT;
                     continue;
                 }
-                Color = Colors[CLR_Regexp];
+                Color = CLR_Regexp;
                 goto hilit;
             case hsPerl_Regexp2:
-                Color = Colors[CLR_Regexp];
+                Color = CLR_Regexp;
                 if ((len >= 2) && (*p == '\\')) {
                     goto hilit2;
                 } else if (*p == QCHAR(State)) {
-                    Color = Colors[CLR_RegexpDelim];
+                    Color = CLR_RegexpDelim;
                     ColorNext();
                     /*State = hsPerl_Normal | X_BIT;*/
                     State = QSET(hsPerl_Regexp1, QCHAR(State));
@@ -468,7 +577,7 @@ int Hilit_PERL(EBuffer *BF, int /*LN*/, PCell B, int Pos, int Width, ELine *Line
                 }
                 goto hilit;
             case hsPerl_Regexp2M:
-                Color = Colors[CLR_Regexp];
+                Color = CLR_Regexp;
                 if ((len >= 2) && (*p == '\\')) {
                     goto hilit2;
                 } else if (GET_BRACE(QCHAR(State) & 3) == *p) {
@@ -477,7 +586,7 @@ int Hilit_PERL(EBuffer *BF, int /*LN*/, PCell B, int Pos, int Width, ELine *Line
                 } else if (IS_MBRACE(GET_BRACE(QCHAR(State) & 3), *p)) {
                     State -= 1 << (2 + 8);
                     if ((QCHAR(State) >> 2) == 0) {
-                        Color = Colors[CLR_RegexpDelim];
+                        Color = CLR_RegexpDelim;
                         ColorNext();
                         /*State = hsPerl_Normal | X_BIT;*/
                         State = hsPerl_Regexp1Op;
@@ -487,25 +596,22 @@ int Hilit_PERL(EBuffer *BF, int /*LN*/, PCell B, int Pos, int Width, ELine *Line
                 }
                 goto hilit;
             case hsPerl_Data:
-                Color = Colors[CLR_Comment];
+                Color = CLR_Comment;
                 goto hilit;
             case hsPerl_HereDoc:
-                if (i == 0) {
-                    for (int xx = 0; xx < len; xx++)
-                        if (p[xx] != '_' && (p[xx] < 'A' || p[xx] > 'Z'))
-                            goto not_end;
-                    Color = Colors[CLR_Punctuation];
-                    setHereDoc = QCHAR(State);
-                    while (len > 0)
-                        ColorNext();
-                    State = hsPerl_Normal | (State & X_BIT);
-                    continue;
+                if (!isEOHereDoc)
+                {
+                    Color = CLR_String;
+                    goto hilit;
                 }
-            not_end:
-                Color = Colors[CLR_String];
-                goto hilit;
+                Color = CLR_Punctuation;
+                setHereDoc = QCHAR(State);
+                while (len > 0)
+                    ColorNext();
+                State = hsPerl_Normal | (State & X_BIT);
+                continue;
             case hsPerl_Docs:
-                Color = Colors[CLR_Comment];
+                Color = CLR_Comment;
                 if (i == 0 && *p == '=' && len > 3 &&
                     p[1] == 'c' && p[2] == 'u' && p[3] == 't')
                 {
@@ -514,15 +620,15 @@ int Hilit_PERL(EBuffer *BF, int /*LN*/, PCell B, int Pos, int Width, ELine *Line
                     ColorNext();
                     ColorNext();
                     State = hsPerl_Normal;
-                    Color = Colors[CLR_Normal];
+                    Color = CLR_Normal;
                     continue;
                 }
                 goto hilit;
             case hsPerl_Comment:
-                Color = Colors[CLR_Comment];
+                Color = CLR_Comment;
                 goto hilit;
             case hsPerl_String1:
-                Color = Colors[CLR_String];
+                Color = CLR_String;
                 if ((len >= 2) && (*p == '\\')) {
                     goto hilit2;
                 } else if (*p == QCHAR(State)) {
@@ -532,7 +638,7 @@ int Hilit_PERL(EBuffer *BF, int /*LN*/, PCell B, int Pos, int Width, ELine *Line
                 }
                 goto hilit;
             case hsPerl_String2:
-                Color = Colors[CLR_String];
+                Color = CLR_String;
                 if ((len >= 2) && (*p == '\\')) {
                     goto hilit2;
                 } else if (*p == QCHAR(State)) {
@@ -542,7 +648,7 @@ int Hilit_PERL(EBuffer *BF, int /*LN*/, PCell B, int Pos, int Width, ELine *Line
                 }
                 goto hilit;
             case hsPerl_StringBk:
-                Color = Colors[CLR_String];
+                Color = CLR_String;
                 if ((len >= 2) && (*p == '\\')) {
                     goto hilit2;
                 } else if (*p == QCHAR(State)) {
