@@ -17,7 +17,7 @@ GxView::GxView(GFrame *Parent): GView(Parent, -1, -1) {
 GxView::~GxView() {
     if (Top) {
         ExView *V;
-        
+
         while (Top) {
             V = Top;
             Top = Top->Next;
@@ -30,7 +30,7 @@ GxView::~GxView() {
 void GxView::PushView(ExView *view) {
     int W, H;
     ConQuerySize(&W, &H);
-    
+
     view->Win = this;
     if (Top == 0) {
         Top = Bottom = view;
@@ -46,15 +46,15 @@ void GxView::PushView(ExView *view) {
 
 ExView *GxView::PopView() {
     ExView *V;
-    
+
     if (Top == 0)
         return 0;
-    
+
     Top->Activate(0);
-    
+
     V = Top;
     Top = Top->Next;
-    
+
     if (Top == 0)
         Bottom = 0;
     else {
@@ -62,11 +62,11 @@ ExView *GxView::PopView() {
         Top->Repaint();
     }
     V->Win = 0;
-    
+
     return V;
 }
 
-void GxView::NewView(ExView *view) {
+void GxView::NewView(ExView * /*view*/) {
 }
 
 EEventMap *GxView::GetEventMap() {
@@ -97,12 +97,12 @@ int GxView::GetContext() {
 void GxView::HandleEvent(TEvent &Event) {
     GView::HandleEvent(Event);
     Top->HandleEvent(Event);
-    
+
     if (Event.What & evMouse) {
         int W, H;
-        
+
         ConQuerySize(&W, &H);
-        
+
         if (Event.What != evMouseDown || Event.Mouse.Y == H - 1) {
             switch (Event.What) {
             case evMouseDown:
@@ -161,7 +161,7 @@ void GxView::Resize(int width, int height) {
     ExView *V;
     GView::Resize(width, height);
     V = Top;
-    
+
     while (V) {
         V->Resize(width, height);
         V = V->Next;
@@ -180,35 +180,50 @@ void GxView::UpdateTitle(char *Title, char *STitle) {
     }
 }
 
-int GxView::ReadStr(char *Prompt, unsigned int BufLen, char *Str, Completer Comp, int Select, int HistId) {
+int GxView::GetStr(const char *Prompt, unsigned int BufLen, char *Str, int HistId) {
+    if ((HaveGUIDialogs & GUIDLG_PROMPT) && GUIDialogs) {
+        return DLGGetStr(this, Prompt, BufLen, Str, HistId, 0);
+    } else {
+        return ReadStr(Prompt, BufLen, Str, 0, 1, HistId);
+    }
+}
+
+int GxView::GetFile(const char *Prompt, unsigned int BufLen, char *Str, int HistId, int Flags) {
+    if ((HaveGUIDialogs & GUIDLG_FILE) && GUIDialogs)
+        return DLGGetFile(this, Prompt, BufLen, Str, Flags);
+    else
+        return ReadStr(Prompt, BufLen, Str, CompletePath, SelectPathname, HistId);
+}
+
+int GxView::ReadStr(const char *Prompt, unsigned int BufLen, char *Str, Completer Comp, int Select, int HistId) {
     int rc;
     ExInput *input;
-    
+
     input = new ExInput(Prompt, Str, BufLen, Comp, Select, HistId);
     if (input == 0)
         return 0;
-    
+
     PushView((ExView *)input);
-    
+
     rc = Execute();
-    
+
     PopView();
-    
+
     Repaint();
-    
+
     if (rc == 1) {
         strncpy(Str, input->Line, BufLen - 1);
         Str[BufLen - 1] = 0;
     }
     delete input;
-    
+
     return rc;
 }
 
-int GxView::Choice(unsigned long Flags, char *Title, int NSel, ... /* choices, format, args */) {
+int GxView::Choice(unsigned long Flags, const char *Title, int NSel, ... /* choices, format, args */) {
     int rc;
     va_list ap;
-    
+
     if ((HaveGUIDialogs & GUIDLG_CHOICE) && GUIDialogs) {
         va_start(ap, NSel);
         rc = DLGPickChoice(this, Title, NSel, ap, Flags);
@@ -216,45 +231,45 @@ int GxView::Choice(unsigned long Flags, char *Title, int NSel, ... /* choices, f
         return rc;
     } else {
         ExChoice *choice;
-        
+
         va_start(ap, NSel);
-        
+
         choice = new ExChoice(Title, NSel, ap);
-        
+
         va_end(ap);
-        
+
         if (choice == 0)
             return 0;
-        
+
         PushView(choice);
         rc = Execute();
         PopView();
         Repaint();
-        
+
         delete choice;
-        
+
         return rc;
     }
 }
 
-TKeyCode GxView::GetChar(char *Prompt) {
+TKeyCode GxView::GetChar(const char *Prompt) {
     int rc;
     ExKey *key;
     TKeyCode K = 0;
-    
+
     key = new ExKey(Prompt);
     if (key == 0)
         return 0;
-    
+
     PushView(key);
     rc = Execute();
     PopView();
     Repaint();
-    
+
     if (rc == 1)
         K = key->Key;
     delete key;
-    
+
     return K;
 }
 
@@ -263,24 +278,24 @@ int GxView::IncrementalSearch(EView *View) {
     int rc;
     ExISearch *search;
     EBuffer *B = 0;
-    
+
     if (View->GetContext() != CONTEXT_FILE)
         return 0;
-    
+
     B = (EBuffer *)View->Model;
-    
+
     search = new ExISearch(B);
-    
+
     if (search == 0)
         return 0;
-    
+
     PushView(search);
     rc = Execute();
     PopView();
     Repaint();
-    
+
     delete search;
-    
+
     return rc;
 }
 #endif
@@ -289,33 +304,35 @@ int GxView::IncrementalSearch(EView *View) {
 int GxView::PickASCII() {
     int rc;
     ExASCII *ascii;
-    
+
     ascii = new ExASCII();
     if (ascii == 0)
         return 0;
-    
+
     PushView(ascii);
     rc = Execute();
     PopView();
     Repaint();
-    
+
     delete ascii;
     return rc;
 }
 #endif
 
-int GxView::GetFile(char *Prompt, unsigned int BufLen, char *Str, int HistId, int Flags) {
-    if ((HaveGUIDialogs & GUIDLG_FILE) && GUIDialogs) {
-        return DLGGetFile(this, Prompt, BufLen, Str, Flags);
-    } else {
-        return ReadStr(Prompt, BufLen, Str, CompletePath, SelectPathname, HistId);
-    }
-}
+#ifdef CONFIG_I_COMPLETE
+int GxView::ICompleteWord(EView *View) {
+    int rc = 0;
 
-int GxView::GetStr(char *Prompt, unsigned int BufLen, char *Str, int HistId) {
-    if ((HaveGUIDialogs & GUIDLG_PROMPT) && GUIDialogs) {
-        return DLGGetStr(this, Prompt, BufLen, Str, HistId, 0);
-    } else {
-        return ReadStr(Prompt, BufLen, Str, 0, 1, HistId);
+    if (View->GetContext() == CONTEXT_FILE) {
+        ExComplete *c = new ExComplete((EBuffer *)View->Model);
+        if (c != NULL) {
+            PushView(c);
+            rc = Execute();
+            PopView();
+            Repaint();
+            delete c;
+        }
     }
+    return rc;
 }
+#endif
