@@ -31,6 +31,7 @@ public:
     
     virtual void DrawLine(PCell B, int Line, int Col, ChColor color, int Width);
     virtual char *FormatLine(int Line);
+    virtual int IsHilited(int Line);
 
     virtual void RescanList();
 //    virtual void UpdateList();
@@ -44,8 +45,9 @@ public:
     virtual void GetTitle(char *ATitle, int MaxLen, char *ASTitle, int SMaxLen);
     
     int isDir(int No);
-    int FmChDir(const char *Name);
-    int FmLoad(char *Name, EView *View);
+    int FmChDir(const char* Name);
+    int FmLoad(const char* Name, EView *View);
+    int FmRmDir(const char* Name);
     int ChangeDir(ExState &State);
     int RescanDir();
 };
