@@ -16,6 +16,7 @@ typedef struct {
     int line;
     char *msg;
     char *text;
+    int hilit;
     EBuffer *Buf;
 } Error;
 
@@ -57,15 +58,16 @@ public:
     virtual int ExecCommand(int Command, ExState &State);
 
     void AddError(Error *p);
-    void AddError(char *file, int line, char *msg, const char *text);
+    void AddError(char *file, int line, char *msg, const char *text, int hilit=0);
     
     void FreeErrors();
-    int GetLine(char *Line, int max);
+    int GetLine(char *Line, int maxim);
     void GetErrors();
     int Compile(char *Command);
     void ShowError(EView *V, int err);
     void DrawLine(PCell B, int Line, int Col, ChColor color, int Width);
     char* FormatLine(int Line);
+    int IsHilited(int Line);
     void UpdateList();
     int Activate(int No);
     int CanActivate(int Line);
@@ -74,6 +76,8 @@ public:
     virtual void GetInfo(char *AInfo, int MaxLen);
     virtual void GetPath(char *APath, int MaxLen);
     virtual void GetTitle(char *ATitle, int MaxLen, char *ASTitle, int SMaxLen);
+    virtual int GetRowLength(int ARow);
+
 
     int RunPipe(char *Dir, char *Command);
     
@@ -82,6 +86,9 @@ public:
 };
 
 extern EMessages *CompilerMsgs;
+
+void FreeCRegexp();
+
 #endif
 
 #endif
