@@ -12,44 +12,50 @@
 EModel *ActiveModel = 0;
 char msgbuftmp[MSGBUFTMP_SIZE] = "";
 
-int GetNewModelID(EModel *B) {
-    static int lastid = 0;
+EModel *FindModelID(EModel *Model, int ID) {
+    EModel *M = Model;
+    int No = ID;
 
-    return lastid++;
-
-    /*EModel *p = B;
-    int m = 1;
-    int did = 0;
-    
-    while (p) {
-        if (p->ModelNo == m) {
-            m++;
-            did = 1;
-        }
-        p = p->Next;
-        if (p == B)
-            if (!did)
-                break;
-            else
-                did = 0;
+    while (M) {
+        if (M->ModelNo == No)
+            return M;
+        M = M->Next;
+        if (M == Model)
+            break;
     }
-    return m;*/
+    return 0;
 }
 
-EModel::EModel(EModel **ARoot) {
+int GetNewModelID(EModel *B) {
+    static int lastid = -1;
+
+    if (ReassignModelIds) lastid = 0;   // 0 is used by buffer list
+    while (FindModelID(B, ++lastid) != 0) /* */;
+
+    return lastid;
+}
+
+EModel::EModel(int createFlags, EModel **ARoot) {
     Root = ARoot;
+
     if (Root) {
         if (*Root) {
-            Next = *Root;
-            Prev = (*Root)->Prev;
+            if (createFlags & cfAppend) {
+                Prev = *Root;
+                Next = (*Root)->Next;
+            } else {
+                Next = *Root;
+                Prev = (*Root)->Prev;
+            }
             Prev->Next = this;
             Next->Prev = this;
         } else
             Prev = Next = this;
-        *Root = this;
-    } else {
+
+        if (!(createFlags & cfNoActivate))
+            *Root = this;
+    } else
         Prev = Next = this;
-    }
     View = 0;
     ModelNo = -1;
     ModelNo = GetNewModelID(this);
@@ -99,18 +105,18 @@ void EModel::SelectView(EView *V) {
     AddView(V);
 }
 
-EViewPort *EModel::CreateViewPort(EView *V) {
+EViewPort *EModel::CreateViewPort(EView * /*V*/) {
     return 0;
 }
 
-int EModel::ExecCommand(int Command, ExState &State) {
+int EModel::ExecCommand(int /*Command*/, ExState &/*State*/) {
     return ErFAIL;
 }
 
-void EModel::HandleEvent(TEvent &Event) {
+void EModel::HandleEvent(TEvent &/*Event*/) {
 }
 
-void EModel::Msg(int level, char *s, ...) {
+void EModel::Msg(int level, const char *s, ...) {
     va_list ap;
     
     if (View == 0)
@@ -120,7 +126,7 @@ void EModel::Msg(int level, char *s, ...) {
     vsprintf(msgbuftmp, s, ap);
     va_end(ap);
     
-    if (level != BUSY)
+    if (level != S_BUSY)
         View->SetMsg(msgbuftmp);
 }
 
@@ -128,34 +134,34 @@ int EModel::CanQuit() {
     return 1;
 }
 
-int EModel::ConfQuit(GxView *V) {
+int EModel::ConfQuit(GxView * /*V*/, int /*multiFile*/) {
     return 1;
 }
 
 int EModel::GetContext() { return CONTEXT_NONE; }
 EEventMap *EModel::GetEventMap() { return 0; }
 int EModel::BeginMacro() { return 1; }
-void EModel::GetName(char *AName, int MaxLen) { *AName = 0; }
-void EModel::GetPath(char *APath, int MaxLen) { *APath = 0; }
-void EModel::GetInfo(char *AInfo, int MaxLen) { *AInfo = 0; }
-void EModel::GetTitle(char *ATitle, int MaxLen, char *ASTitle, int SMaxLen) { *ATitle = 0; *ASTitle = 0; }
-void EModel::NotifyPipe(int PipeId) { }
+void EModel::GetName(char *AName, int /*MaxLen*/) { *AName = 0; }
+void EModel::GetPath(char *APath, int /*MaxLen*/) { *APath = 0; }
+void EModel::GetInfo(char *AInfo, int /*MaxLen*/) { *AInfo = 0; }
+void EModel::GetTitle(char *ATitle, int /*MaxLen*/, char *ASTitle, int /*SMaxLen*/) { *ATitle = 0; *ASTitle = 0; }
+void EModel::NotifyPipe(int /*PipeId*/) { }
 
-void EModel::NotifyDelete(EModel *Deleted) {
+void EModel::NotifyDelete(EModel * /*Deleted*/) {
 }
 void EModel::DeleteRelated() {
 }
 
 EViewPort::EViewPort(EView *V) { View = V; ReCenter = 0; }
 EViewPort::~EViewPort() {}
-void EViewPort::HandleEvent(TEvent &Event) { }
+void EViewPort::HandleEvent(TEvent &/*Event*/) { }
 void EViewPort::UpdateView() { }
 void EViewPort::RepaintView() { }
 void EViewPort::UpdateStatus() { }
 void EViewPort::RepaintStatus() { }
 void EViewPort::GetPos() { }
 void EViewPort::StorePos() { }
-void EViewPort::Resize(int Width, int Height) {}
+void EViewPort::Resize(int /*Width*/, int /*Height*/) {}
 
 void EModel::UpdateTitle() {
     char Title[256] = ""; //fte: ";
@@ -171,3 +177,15 @@ void EModel::UpdateTitle() {
         V = V->NextView;
     }
 }
+
+int EModel::GetStrVar(int var, char *str, int buflen) {
+    switch (var) {
+    case mvCurDirectory:
+        return GetDefaultDirectory(this, str, buflen);
+    }
+    return 0;
+}
+
+int EModel::GetIntVar(int /*var*/, int * /*value*/) {
+    return 0;
+}
