@@ -25,29 +25,34 @@ public:
     virtual void RepaintView();
     virtual void UpdateStatus();
     virtual void RepaintStatus();
-    
+
     virtual void GetPos();
     virtual void StorePos();
-    
+
     virtual void Resize(int Width, int Height);
 };
 
+enum createFlags {
+    cfAppend = 1,
+    cfNoActivate = 2
+};
+
 class EModel {
 public:
     EModel **Root;   // root ptr of this list
     EModel *Next;    // next model
     EModel *Prev;    // prev model
     EView *View;     // active view of model
-    
+
     int ModelNo;
-    
-    EModel(EModel **ARoot);
+
+    EModel(int createFlags, EModel **ARoot);
     virtual ~EModel();
 
     void AddView(EView *V);
     void RemoveView(EView *V);
     void SelectView(EView *V);
-    
+
     virtual EViewPort *CreateViewPort(EView *V);
 
     virtual int GetContext();
@@ -64,12 +69,15 @@ public:
 
     void UpdateTitle();
 
-    void Msg(int level, char *s, ...);
+    void Msg(int level, const char *s, ...);
     virtual int CanQuit();
-    virtual int ConfQuit(GxView *V);
-    
+    virtual int ConfQuit(GxView *V, int multiFile = 0);
+
+    virtual int GetStrVar(int var, char *str, int buflen);
+    virtual int GetIntVar(int var, int *value);
+
     virtual void NotifyPipe(int PipeId);
-    
+
     virtual void NotifyDelete(EModel *Deleting);
     virtual void DeleteRelated();
 };
@@ -89,26 +97,26 @@ public:
 
     virtual void FocusChange(int GotFocus);
     virtual void Resize(int Width, int Height);
-    
+
     void SetModel(EModel *AModel);
     void SelectModel(EModel *AModel);
     void SwitchToModel(EModel *AModel);
-    
+
     void Activate(int GotFocus);
-    
+
     virtual int GetContext();
     virtual EEventMap *GetEventMap();
     virtual int BeginMacro();
     virtual int ExecCommand(int Command, ExState &State);
-    
+
     virtual void HandleEvent(TEvent &Event);
     virtual void UpdateView();
     virtual void RepaintView();
     virtual void UpdateStatus();
     virtual void RepaintStatus();
-    
-    void Msg(int level, char *s, ...);
-    void SetMsg(char *Msg);
+
+    void Msg(int level, const char *s, ...);
+    void SetMsg(const char *msg);
 
     int SwitchTo(ExState &State);
     int FilePrev();
@@ -126,9 +134,27 @@ public:
 #endif
 #ifdef CONFIG_OBJ_MESSAGES
     int Compile(ExState &State);
+    int RunCompiler(ExState &State);
+    int Compile(char *Command);
     int ViewMessages(ExState &State);
     int CompilePrevError(ExState &State);
     int CompileNextError(ExState &State);
+    int ConfigRecompile(ExState &State);
+#endif
+#ifdef CONFIG_OBJ_CVS
+    int Cvs(ExState &State);
+    int RunCvs(ExState &State);
+    int ViewCvs(ExState &State);
+    int Cvs(char *Options);
+    int ClearCvsMessages(ExState &State);
+    int CvsDiff(ExState &State);
+    int RunCvsDiff(ExState &State);
+    int ViewCvsDiff(ExState &State);
+    int CvsDiff(char *Options);
+    int CvsCommit(ExState &State);
+    int RunCvsCommit(ExState &State);
+    int CvsCommit(char *Options);
+    int ViewCvsLog(ExState &State);
 #endif
 #ifdef CONFIG_OBJ_DIRECTORY
     int DirOpen(ExState &State);
@@ -140,14 +166,25 @@ public:
 #ifdef CONFIG_TAGS
     int TagLoad(ExState &State);
 #endif
-    
+    int SysShowHelp(ExState &State, const char *word);
+
+    int RemoveGlobalBookmark(ExState &State);
+    int GotoGlobalBookmark(ExState &State);
+    int PopGlobalBookmark();
+
     void DeleteModel(EModel *M);
     int CanQuit();
+
+    int GetStrVar(int var, char *str, int buflen);
+    int GetIntVar(int var, int *value);
 };
 
 extern EModel *ActiveModel;
 extern EView *ActiveView;
 
+int GetNewModelID(EModel *B);
+EModel *FindModelID(EModel *B, int ID);
+
 #define MSGBUFTMP_SIZE 1024
 extern char msgbuftmp[MSGBUFTMP_SIZE];
 
