@@ -146,8 +146,17 @@ int IsDirectory(const char *Path) {
         else
 #endif
             if (!ISSLASH(Path[len - 1])) {
+                int res;
                 struct stat statbuf;
-                if (stat(Path, &statbuf) == 0) {
+#if PATHTYPE == PT_UNIXISH
+                if (Path[0] == '~') {
+                    char Expanded[MAXPATH];
+                    if (ExpandPath(Path, Expanded) == -1) return 0;
+                    res = stat(Expanded, &statbuf);
+                } else
+#endif
+                    res = stat(Path, &statbuf);
+                if (res == 0) {
                     if (S_ISDIR(statbuf.st_mode)) {
                         return 1;
                     }
@@ -164,7 +173,7 @@ static int GetDiskCurDir(int drive, char *dir) {
     if (drive < 1 || drive > 26)
         return -1;
 
-#if defined(EMX)
+#if defined(__EMX__)
     return (_getcwd1(dir, (char)(drive + 'A' - 1)) == 0) ? 0 : -1;
 #elif defined(OS2)
     {
@@ -204,7 +213,7 @@ static int SetDrive(int drive) { // 1 = A, 2 = B, 3 = C, ...
     // _chdrive seems to be most portable between dosish systems,
     // but seem to take different arguments ???
 
-#if defined(EMX)
+#if defined(__EMX__)
     return _chdrive(drive + 'A' - 1);
 #elif defined(OS2)
     return (DosSetDefaultDisk(drive) == 0) ? 0 : -1;
@@ -286,14 +295,14 @@ int ExpandPath(const char *Path, char *Expand) {
         if (GetDiskCurDir(drive, Expand + 3) == -1)
             return -1;
     } else {
-#if defined(EMX)
+#if defined(__EMX__)
         if (_fullpath(Expand, Name, MAXPATH) == -1) return -1;
 #else
         if (_fullpath(Expand, Name, MAXPATH) == NULL) return -1;
 #endif
     }
 #endif
-#if defined(EMX)
+#if defined(__EMX__)
     {
         char *p = Expand;
 
@@ -319,7 +328,8 @@ int ExpandPath(const char *Path, char *Expand) {
     case '~':
         if (Name[1] == SLASH || Name[1] == 0) {
             path = Name + 1;
-            strcpy(Name2, getenv("HOME"));
+	    strncpy(Name2, getenv("HOME"), sizeof(Name2) - 1);
+            Name2[sizeof(Name2) - 1] = 0;
         } else {
             struct passwd *pwd;
 
@@ -372,6 +382,19 @@ int JustDirectory(const char *Path, char *Dir) {
     return 0;
 }
 
+int JustLastDirectory(const char *Path, char *Dir) {
+    int lastSlash = strlen(Path);
+    while (lastSlash > 0 && !ISSEP(Path[lastSlash])) lastSlash--;
+
+    int secondLastSlash = lastSlash;
+    while (secondLastSlash > 0 && !ISSEP(Path[secondLastSlash - 1])) secondLastSlash--;
+
+    strncpy(Dir, Path + secondLastSlash, lastSlash - secondLastSlash);
+    Dir[lastSlash - secondLastSlash] = 0;
+
+    return 0;
+}
+
 int JustFileName(const char *Path, char *Name) {
     int len = strlen(Path);
 
@@ -453,3 +476,24 @@ char *SepRChr(char *Dir)
     }
     return NULL;
 }
+
+// Returns relative path of Path with respect to Dir
+// Ex: Dir = /home/martin/src/ Path = /home/martin/src/Fte/s_files.cpp -> Fte/s_files.cpp
+int RelativePathName(const char *Dir, const char *Path, char *RelPath) {
+    char d[MAXPATH], p[MAXPATH], c;
+    int dl, pl;
+
+    if (ExpandPath(Dir, d) == -1) return -1;
+    if (ExpandPath(Path, p) == -1) return -1;
+    dl = strlen (d); pl = strlen (p);
+
+    if (dl <= pl) {
+        c = p[dl]; p[dl] = 0;
+        if (filecmp(d, p) == 0) {
+            p[dl] = c;
+            strcpy(RelPath, p + dl);
+            return 0;
+        }
+    }
+    return -1;
+}
