@@ -40,6 +40,7 @@ int CompletePath(const char *Path, char *Completed, int Num);
 int IsSameFile(const char *Path1, const char *Path2);
 int JustDirectory(const char *Path, char *Dir);
 int JustFileName(const char *Path, char *Name);
+int JustLastDirectory(const char *Path, char *Dir);
 int JustRoot(const char *Path, char *Root);
 int FileExists(const char *Path);
 int IsFullPath(const char *Path);
@@ -48,5 +49,6 @@ const char *ShortFName(const char *Path, int len);
 int ChangeDir(char *Dir);
 int JoinDirFile(char *Dest, const char *Dir, const char *Name);
 char *SepRChr(char *Dir);
+int RelativePathName(const char *Dir, const char *Path, char *RelPath);
 
 #endif
